@mainpage

# Introduction  

This is the WCCOA CTL Libraries documentation. WCCOA CTL Libraries are collection of control(CTL) function for the supervisory application WinCC OA(TM).  

The WCCOA CTL Libraries are part of the component WCCOA_unVacuum (the second part is the Qt Libraries not include in this documentation, they are documented with in another doxygen project.
)

The gitlab project is WCCOA_unVacuum in the group ucpc/Externals/Vacuum/. The repository is `installed_components/scritps/`.

https://gitlab.cern.ch/ucpc/Externals/Vacuum/wccoa_unvacuum

This component is part of the Unicos Vacuum package: [unicos-vacuum.web.cern.ch](https://unicos-vacuum.web.cern.ch/)

# Table of content

- @ref Architecture
- @ref animation 
- @ref menu 
- @ref Faceplate
- @ref state

# References

The developers are: 

- Sebastien Blanchard <Sebastien.Blanchard@cern.ch>
- Leonid Kopylov <Leonid.Kopylov@cern.ch>
- Thijs van Winden <Thijs.van.Winden@cern.ch>
- Marcin Bes <Marcin.Bes@cern.ch>

The documentation related pages are in the file `installed_components/scritps/Readme.md`. 

Documentation is generated by Doxygen using Markdown format for static documentation.

This documentation webpage:

https://unicos-vacuum.web.cern.ch/WCCOA_component/WCCOA/

This documentation EDMS pdf:

https://edms.cern.ch/document/2435044





@page Architecture

# Introduction

This chapter explains the general structure of a SCADA object, which consists of the importation and animation scripts, panels and datapoint types, datapoints and configuration. 

# Datapoints

For every object a datapoint type has to be created defining the general I/O 
and status information. Normally the developer first defines the general 
structure in the TCT application and will use this to define the datapoint in 
WinCC-OA. 

A very simple example of a datapoint object would be:

~~~c

    CPC_DeviceName
	statusInformation:      _UnStatusInformation
	ProcessInput
		StsReg01:       UINT	
		PosSt:          FLOAT          
		evStsReg01:     UINT			
		ObjectSt:       UINT	
	ProcessOutput	
		ManReg01:       UINT	
		MPosR:          UINT	
	eventMask:              UINT //To be checked
	disabled:               BOOL	

~~~

The `statusInformation` as instance of `UnStatusInformation` is holding 
information for UNICOS, like selected state, device links, access control, 
device logging and more configuration. All these are configured per object 
and not per object type and mostly coming from the importation. More info in the 
section about `Importation`.

Besides this datapoint type also a few other datapoints are defined for UNICOS, for vacuum saved in `VACUUMfwDeviceDefinition` and `VACUUMinfo`. The first `VACUUMfwDeviceDefinition` contains `CPC_VX_XXInfo` of datapoint type `_FwDeviceDefinition`. The definition is omited for brevity, but here things can
be defined that hold datapoint wide. 

Finally `VACUUMinfo` contains the datatpoints for each objects of types `_UnObjects`,  `_UnTrendConfiguration`, `FwTrendingPlot`. The `CPC_OnOff_unicosInfo` of type `_UnObjects` is used to configure the possible widgets, faceplates, callback functions and more. It's encouraged for the user to have a look at these files. 

The datapoint `UnTrendConfiguration` and `FwTrendingPlot` are used to configure the 
UNICOS trending. Since this is not used in the vacuum package, this won't be 
elaborated on.

The next step after defining the datapoint would be defining the import script.

# Importation

In UNICOS UAB generates a `csv` file containing all the data to be imported for an objects. The object 
itself contains a script parsing this data and configuring the object, `CPC_DeviceTypeConfig.ctl`. 

The importation works in a bit of peculiar manner. For every object a set of constants is defined in 
this script. This value of these constants is referring to a certain element at a certain position in 
the CSV. The name of the constant is formed according to a standard that UNICOS follows.

For example:

~~~c

   const UN_CONFIG_CPC_VG_PT_UNIT = 1;

~~~

It would refer to element 11. Why 11? UNICOS assumes that the first 10 elements are fixed and used internally
by UNICOS to define the object.

UNICOS relates the name to value that needs to be imported. If the name is analyzed one can see that 
we can divide it up in three pieces. Prefix `UN_CONFIG`, Object type `CPC_VG_PT` and actual meaning UNIT.

If no DPE is provided it is assumed to be applied on the PosSt. Thus `UN_CONFIG_CPC_VG_PT_UNIT` and `UN_CONFIG_CPC_VG_PT_POSST_UNIT` are identical.

In fact this holds for the following list of constants:

~~~c

 UN_CONFIG_CPC_VG_PT_ADDITIONAL_POSST_UNIT
 UN_CONFIG_CPC_VG_PT_ADDITIONAL_UNIT
 UN_CONFIG_CPC_VG_PT_POSST_UNIT (Recommended in the unVacuum package)
 UN_CONFIG_CPC_VG_PT_UNIT

~~~

This list is searched in the order it's displayed. In this manual the term `UNIT` will refer to one of the 
constants above. It's up to the user to chose the proper one.

Many more constants can be defined and it would be best to have a look at the existing import scripts. Also
the function `:func:cpcConfigGenericFunctions_setDeviceConfig` and under laying methods
like `:func:cpcConfigGenericFunctions_setFormat` shows a lot of the options. 


For the actual naming convention for the constants have a look at `:func:cpcConfigGenericFunctions_getConstantDPEProperty`. Beside the constants various functions are defined in this script. In this manual we will quickly look into each of  the important functions during the import.

~~~c

  function: mapping CPCDeviceTypeConfig_getConfig()

~~~

Defines the configuration for the DPE's of this object. This configuration is related with archiving, datatypes, addressing, units etc. 
    
~~~c

    return: A hashmap containing the configuration for the different DPE's representing the input and output

~~~

A very simple example for random object `VX_XX` is the following. 

~~~c

    mapping CPC_VX_XXConfig_getConfig() {
        mapping config;
        mapping props;

        // Status adresses
        // StsReg01
        mappingClear(props);
        props["address"] = ".ProcessInput.StsReg01";
        props["dataType"] = CPC_UINT16;
        config["StsReg01"] = props;
        // evStsReg01
        mappingClear(props);
        props["hasArchive"] = true;
        props["address"] = ".ProcessInput.evStsReg01";
        props["dataType"] = CPC_INT32;
        config["evStsReg01"] = props;       
        // ObjectSt
        mappingClear(props);
        props["hasMapping"] = true;
        props["hasUnit"] = false;
        props["hasFormat"] = false;
        props["address"] = ".ProcessInput.ObjectSt";
        props["dataType"] = CPC_UINT16;
        config["ObjectSt"] = props;
        
        // Request adresses
        // ManReg01
        mappingClear(props);
        props["address"] = ".ProcessOutput.ManReg01";
        props["dataType"] = CPC_UINT16;
        config["ManReg01"] = props;
      
        return config;
    }

~~~

As can be seen here various properties can be used to configure the import. 

~~~c

address
    The actual datapoint element name in WinCC-OA without the device name, i.e. .ProcessInput.StsReg01
dataType
    The datatype as defined in the CPC constants. For all options see :func: cpcConfigGenericFunctions_translateDataType
hasUnit
    Whether the DPE defined in the address has a unit defined in the constants REF. Refers to the _UNIT constant
hasFormat
    Whether the DPE defined in the address has a format defined in the constants REF. Refers to the _FORMAT constant
isAlarm
    Whether the DPE defined in the address is an alarm signal. Should always come with the bitPosition property
bitPosition
    Makes the relation between a DPE and a bit in the StsReg
hasSmooth
    True when smoothening needs to be applied to the DPE. Refers to the _DEADBAND and _DEADBAND_TYPE constants
hasPvRange
    Set an upper and lower boundary for this element. Refers to the _RANGEMAX and _RANGEMIN constants
hasMapping
    Wheter a mapping is defined for this variable. This maps integer values to a string defined in a dictionary. 
    Mappings should be defined in the following format: 1=Status Ok,2=Error,3=..,etc. Refers to the _MAPPING property
hasArchive
    Configure whether this DPE has archiving. Refers to the _ARCHIVE_ACTIVE and _ARCHIVE_TIME_FILTER constants
5RangesAlerts
    Special UNICOS alert type. For more information check the code of the 'VG_PT'

~~~

~~~c

.. function: dyn_string CPC_DeviceTypeConfig_getParamNames();

    Defines a list parameter names, which will be automatically saved by UNICOS in the 
    
    :return: A list of names corresponding with the parameters defined in the parameters 
        field in the specs and the once added by the python scripts
   
    dyn_string CPC_VR_AHConfig_getParamNames() {
        // The parameters is built up like "SOURCE1,SOURCE2,1" and
        return makeDynString(CPC_PROP_NAME_SOURCE1, CPC_PROP_NAME_SOURCE2, CPC_PROP_NAME_CONFIG_TYPE);
    }

~~~

~~~c

.. function: CPC_DeviceTypeConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo);
    
    Check if the custom configuration is valid, so everything which is not handled in the constants

    :param dyn_string configLine: The config line splitted up into a list of elements
    :param bool hasArchive: Boolean determining whether this object has archiving or not 
    :param dyn_string exceptionInfo: Value in which any exception information can be stored from underlaying functions or generetated by the function itself

~~~

Examples things that can be checked are data type, ranges, in list of values etc. A simple example of an integer check is shown below.

More builtin functions can be found `unConfigGenericFunctions_check`

~~~c

    CPC_VX_XXConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
        // Check the validity of the 4 relay sources
        unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1SRC_DEFAULT], iValue, exceptionInfo);
        unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2SRC_DEFAULT], iValue, exceptionInfo);    
        unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3SRC_DEFAULT], iValue, exceptionInfo);
        unConfigGenericFunctions_checkInt(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4SRC_DEFAULT], iValue, exceptionInfo);



.. function: CPC_DeviceTypeConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo);

    Actual saving of the custom configuration, not being handled by the constants system. This is 
    can also be used for setting static configuration for this particular type of object.
    
    :param dyn_string configLine: The config line splitted up into a list of elements
    :param bool hasArchive: Boolean determining whether this object has archiving or not 
    :param dyn_string exceptionInfo: Value in which any exception information can be stored from underlaying functions or generetated by the function itself

~~~

Finally we can use this function set some custom configuration data. This can be defaults, but also standard mappings, units, archive settings or even custom 
parameters based on some logic.

~~~c

    CPC_VX_XXConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {

        dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel1Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL1SRC_DEFAULT]);
        dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel2Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL2SRC_DEFAULT]);
        dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel3Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL3SRC_DEFAULT]);
        dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DRel4Src", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VR_PI_REL4SRC_DEFAULT]);

    }

~~~

One special things needs more attention. The handling of the acknowledgements of the alarms this has to be explicitly 
done in the configuration. Without this it won't work. This results in the following snipped inside the 
custom configuration function. 

~~~c

    CPC_VX_XXConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
        // Fix the acknowlegement after the Temperorary Stop, Full stop, Start Interlock and Alarm status
        cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.TStopISt");
        cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.FuStopISt");
        cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.StartISt");
        cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.AlSt");
        
    }

~~~

Once the import script has been defined, this means tested and the user 
successfully manages to import and create a datapoint in WinCCOA, we can start 
with the animation of the actual object.

## Animation script

The animation logic is all defined in the `CPC_DeviceName.ctl` file. Here 
code for the widget, contextual menu, faceplate, state history and alarms can be
found. For the structure we will only quickly zoom in on the widget and general
state animation. This general state animation is used in various extension made 
by the vacuum framework, like the state history and Pressure Profile. 
    
## Widget and State History

To enforce coherency in the color animation of the widget, state history, device 
list, etc it was chosen to combine the calculation of the color for the general 
state and the widget animation. This also reduces the code duplication in the 
libraries. Therefor a few common methods have been defined that every vacuum 
object will support. 

~~~c

.. function: void CPC_DeviceType_StateDPEs(string sDeviceName, dyn_string &dsDpes, [bool bForHistory = true]) 

    Lists the dpes required for building the state of the device
  
    :param string sDeviceName: The name of the device which is in UNICOS the same as the dpname 
    :param dyn_string dsDpes: The list of datapoints elements required for building the state 
    :param bool bForHistory: The DPEs will be used for state history, i.e. only DPEs  
        with archive config are of interest



.. function: int CPC_DeviceType_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, string &sState, bool bUseOffColor, [bool bBoolAlert = false], [bool bIsRestartNeeded = false])
  
    Returns the main color associated with the device state. This color is based on 
    the dpes listed in the :func:CPC_DeviceType_StateDPEs for the dedicated device type. In case of the 
    state history the EvStsReg## is used because this is archived, while for determining 
    the current state the StsReg## is used.
    
    :param string sDeviceName: device dp name to get objectSt meaning
    :param dyn_string dsDpes: The list of dpe names used to determine the color
    :param dyn_anytype daValues: The list of dpe values used to determine the color 
    :param string sColor: the main color string of device
    :param string sString: the main state string of device
    :param bool bUseOffColor: Off color is return in &sColor when device is Off, false in case of WidgetColor
    :param bool bBoolAlert: realtime alert according to UNICOS alert manager
    :param bool bIsRestartNeeded: there is a restart needed according to UNICOS device manager
    
    :return: '0' when function proceed successfully or '-1' when function proceed with error

~~~

The standard UNICOS widget animation function `func:CPC_DeviceType_WidgetAnimation`
will use the color determined by the `func:CPC_DeviceType_ColorAndString` function 
to avoid code duplication. These functions will also be used by the pressure
profile, device list and other vacuum extensions to display the state of the 
device.






@page animation Widget Animation

This chapter will do a very brief recap of the UCPC widget animation structure 
and explain the vacuum extension on top of that.

The normal UCPC way of animating widgets will be used. This means using the 
function `CPC_<DeviceType>_WidgetDPEs` to determine the datapoint elements for the animation and 
the function `CPC_<DeviceType>_WidgetAnimation` for the actual animation. 

The actual animation can be divided into two parts. One for the animation of the 
letters and one for the animation of the body. To reduce code duplication again
a few common methods have been defined to ease the widget animation. These methods
will be discussed with an widget animation example, in this case the `VP_TP`.

# Datapoint elements

To start with, for every object the DPE needed for the animation have to registered in the animation function.
In the standard UCPC this would look something like this. 

~~~c


    void CPC_VP_TP_WidgetDPEs(string deviceName, dyn_string &dpes) {
        dynAppend(dpes, deviceName + ".eventMask");                               //value[offset + 1]
        dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                   //value[offset + 2]
        dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid"); //value[offset + 3]
        dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                   //value[offset + 4]
        dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid"); //value[offset + 5]
        dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                   //value[offset + 6]
        dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid"); //value[offset + 7]
        dynAppend(dpes, deviceName + ".disabled");                                //value[offset + 8]

        if (g_bBoolAlert) {
            dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  //value[offset + 9]
            dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  //value[offset +10]
            dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); //value[offset +11]
            dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      //value[offset +12]
        }
    }

~~~

As one can see here, we basically build a list of DPE. Changes in the values of
will trigger a callback to :func:`CPC_VP_TP_WidgetAnimation`, which will get these 
names and values as arguments alongside the widget type.

~~~

NOTE:
    The widget type is defined in the widget panels `ScopeLib` in the variable `g_sWidgetType`

~~~

Due to the state history extension and to prevent double code it would be 
beneficial to declare these DPE's only one time as both systems share the same 
animation code. Therefor in the vacuum package the following structure is 
recommended

~~~c


    CPC_VP_TP_WidgetDPEs(string deviceName, dyn_string &dpes) {
        // Add the event mask only here cause we don't need in the StateHistory
        dynAppend(dpes, deviceName + ".eventMask");
        // Add the state dpe's needed for the color animation
        CPC_VP_TP_StateDPEs(deviceName, dpes);
        
        // Add the alert DPE's if the alert was set
        if (g_bBoolAlert) {
            dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  //value[offset + 9]
            dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  //value[offset +10]
            dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); //value[offset +11]
            dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      //value[offset +12]
        }
    }

~~~

Once the DPE's have been registered it's time for the animation function itself. 
This guide assumes basic UNICOS and CPC knowledge. The first thing in the 
are the widget letters.

## Letters Animation

### Definition

There are two sets of letters defined for the vacuum widget. The extended UCPC set
and a reduced set of VACUUM letters. The reduced set, is mainly to ignore any 
unsupported functionality in the vacuum objects. They can be divided into three
categories:

* Warning letters
* Alarm letters
* Control letters

The letters are defined as priority lists with an list of bits associated with 
them. The idea is that the letter with the highest priority will be shown if 
the bit is high.

The letters in the three categories are defined like:

Nb  |   CPC_WARNING_BITS       |   VAC_WARNING_BITS      |   CPC_WARNING_LETTERS (Constants)      |    Letters 
----|--------------------------|-------------------------|----------------------------------------|--------
1   |  CPC_StsReg01_IOSIMUW    |  CPC_StsReg01_IOSIMUW   |  CPC_WIDGET_TEXT_WARNING_SIMU          |   S       
2   |  CPC_StsReg01_IOERRORW   |  CPC_StsReg01_IOERRORW  |  CPC_WIDGET_TEXT_WARNING_ERROR         |   E       
3   |  CPC_StsReg01_MIOERBRST  |                         |  CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED |   B       
4   |  CPC_StsReg02_ALBW       |                         |  CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED |   B       
5   |                          |                         |  CPC_WIDGET_TEXT_WARNING_DEFAULT       |   W       


Nb  |   CPC_ALARM_BITS         |   VAC_ALARM_BITS        |   CPC_ALARM_LETTERS (Constants)        |    Letters 
----|  ------------------------|  -----------------------|  --------------------------------------|--------
1   |  CPC_StsReg01_ALST       |                         |  CPC_WIDGET_TEXT_ALARM_ALST            |   A       
2   |  CPC_StsReg01_STARTIST   |                         |  CPC_WIDGET_TEXT_ALARM_STARTIST        |   I       
3   |  CPC_StsReg01_TSTOPIST   |                         |  CPC_WIDGET_TEXT_ALARM_TSTOPIST        |   S       
4   |  CPC_StsReg02_FUSTOPIST  |                         |  CPC_WIDGET_TEXT_ALARM_FUSTOPIST       |   F       
5   |                          |                         |  CPC_WIDGET_TEXT_MANUAL_RESTART        |   R      


Nb  |   CPC_CONTROL_BITS       |   VAC_CONTROL_BITS      |   CPC_CONTROL_LETTERS (Constants)      |    Letters   
----|  ------------------------|-------------------------|----------------------------------------|--------
1   |  CPC_StsReg01_FoMoSt     |                         |  CPC_WIDGET_TEXT_CONTROL_FORCED        |   F      
2   |  CPC_StsReg01_AUIHFOMOST |                         |  CPC_WIDGET_TEXT_CONTROL_AUIHFOMOST    |   h       
3   |  CPC_StsReg01_MaMoSt     |                         |  CPC_WIDGET_TEXT_CONTROL_MANUAL        |   M       
4   |  CPC_StsReg01_AUIHMBST   |                         |  CPC_WIDGET_TEXT_CONTROL_AUINHMMO      |   h

~~~

HINT:
    To clarify the logic above. Let us observe the warning letters. 
    you can see in the table above there are 5 letters defined. If we would use 
    the bit set #CPC_WARNING_BITS# this wold result in the following logic:

    The bit `CPC_StsReg01_IOSIMUW` is high
        The letter `S` is shown
    The bit `CPC_StsReg02_ALBW` is high, but the bit `CPC_StsReg01_IOSIMUW` is low 
        The letter `B` will be shown. 
    Both `CPC_StsReg01_IOSIMUW` and `CPC_StsReg02_ALBW` are high
        The letter `S` will be shown 
    
    This logic applies to all priority lists

~~~

## Letter retrieval

There general method to retrieve the letter is by using the following structure.
First you retrieve the values of the bits as a list of booleans. You can use the
set of bits of your choice here. 

For the warning bits this would look like:

~~~c


    // Create a list of the bit registers
    dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01, bit32StsReg02);
    
    // Fetch all the bits from the registers
    dyn_bool dbBools = vacuumGenericObject_fetchBitsFromStsRegs(dbit32StsRegs, VAC_WARNING_BITS); 

~~~

Then you use this lists of bools, `dbWarningBools` in this example, combined with 
the letter set.

~~~c


    string sLetter;

    // Retrieve the letter using the boolean/letter mapping
    vacuumGenericObject_WidgetBoolMapping(dbBools, CPC_WARNING_LETTERS, sLetter); 

~~~

Ofcourse the letters also have a color and some have some more complicated logic
by default. Therefor a few helper functions have been added to the library for 
the different effects.


### Alarm

The alarm letter has besides the bit based functionality also some logic for the 
restart needed status. This is implemented because some devices need a 
manual reset in case of malfunction. The logic could therfore look like this.

This function will determine the restart needed letter needs to be displayed
and will set the argument sAlarmLetter with the corresponding letter

~~~c

    vacuumGenericObject_WidgetAlarmRestartAnimation(bIsRestartNeeded, sAlarmLetter, sAlarmColor);

~~~

This function will overwrite the restart letter, because of higher priority, 
if another alarm is present.
    
~~~c

		vacuumGenericObject_WidgetAlarmTextAnimation(dbAlarmBits, dsAlarmLetters, sAlarmLetter, sAlarmColor);

~~~

If nothing is encountered the variable sAlarmLetter will be empty and nothing
will be displayed


### Warning

The warning letter is also responsible for showing the invalid and old data state
as normally defined in UCPC. 

First will retrieve any active general warning defined in the letter list

~~~c

    vacuumGenericObject_WidgetWarningTextAnimation(dbWarningBits, dsWarningLetters, sWarningLetter, sWarningColor);    

    // Second if the data is invalid the original warning letter will be overwritten
   
	  cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue, bInvalid, sWarningLetter, sWarningColor);
    
		// If no warning is detected, the letter will be empty and nothing will be displayed

~~~

### Control State

No additional functionality is required for the control state letter. Thus 
it will simply check the bit/letter lists and determine the color

~~~c

    // First will retrieve any active control state defined in the letter list
    vacuumGenericObject_WidgetControlStateAnimation(dbControlBits, dsControlBitLetters, sControlLetter, sControlColor);  
    // If no control state is detected, the letter will be empty and nothing will be displayed
~~~


~~~

NOTE: 
    The select/unselect and lock animation also has to be programmed in the 
    widget animation function. Luckily UCPC provides us with the functions 
    :func:unGenericObject_WidgetSelectAnimation to ease this logic.

~~~

Now this all looks a bit cumbersome to define for every widget. Especially
cause the logic will more or less be the same for every widget, besides some 
small changes. To reduce the code duplication the function 
`vacuumGenericObject_WidgetLetterAnimation` has been added. This will take care
about the general letter detection and also animation. The user will only have to supply the 
letters, booleans and some parameters. An combined example would be this

~~~c


    vacuumGenericObject_WidgetLetterAnimation(sWidgetType, UN_CONFIG_CPC_VP_TP_DPT_NAME,
        bLocked, sSelectedManager, dbit32StsRegs,
        vacuumGenericObject_fetchBitsFromStsRegs(dbit32StsRegs, dsWarningBits),    // Default vacuum warning bits
        CPC_WARNING_LETTERS, // Default vacuum warning letters
        vacuumGenericObject_fetchBitsFromStsRegs(dbit32StsRegs, dsAlarmBits),     // Default vacuum alarm bits
        CPC_ALARM_LETTERS, // Default vacuum alarm letters
        vacuumGenericObject_fetchBitsFromStsRegs(dbit32StsRegs, dsControlBits) ,    // Default vacuum control bits
        CPC_CONTROL_LETTERS, // Default vacuum control letters
        bInvalid, bIsRestartNeeded, bComment,
        iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);

~~~

If the standard animation function doesn't suffice for the new widget, a 
custom one can be implemented by adding the :func:`CPC_<deviceType>_WidgetLetterAnimation`
to the object library. An example is implemented for the `VR_AH`
is: 

~~~c


    // This function is used to prevent the animation of any letters, while at 
    // at the same time still supporting selected and lock animation 
    CPC_VR_AH_WidgetLetterAnimation(string sWarningLetter, string sWarningColor,
        string sAlarmLetter, string sAlarmColor, string sControlLetter, string sControlColor,
	string sColorSelect, bool bLockVisible, bool bComment) {
	
        setMultiValue(
			"SelectArea", "foreCol", sColorSelect
	);
    }

~~~

## Body Animation

The body animation is the second part of the total widget animation. For 
uniformity for the user the colors used in the state history are identical to 
the colors used for the widget body animation. Therefor they share the logic for 
determining the active color. However the datapoints required for the general 
animation function are not always the ones needed for the state history. 

Therefor the following architecture is implemented. The object defines a 
function :func:`CPC_<DeviceType>_ColorAndString` in which the color is determined.
The :func:`CPC_<DeviceType>_WidgetAnimation` will call this function and also
take into account which DPE's this function requires. 

An example of this logic as implemented for `VP_I` would be:

~~~c


  // Determine the body color based on the state

  // Filter the supplied DPE, by checking which indices are not needed for the
  // StateHistory

  dyn_int diKeyFilter = makeDynInt(3, 4, 5, 13, 14); 
  // Filter dpes based on index to be consistent with the StateHistory dpe's
  string sBodyColor, sState;
  dyn_string filteredDpes = vacuumGenericObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = vacuumGenericObject_dynWithoutIndex(values, diKeyFilter);

  // Determine the actual color 
  CPC_VP_I_ColorAndString("", filteredDpes, filteredValues,
      sBodyColor,
      sState,
      false,
      g_bBoolAlert,
      bIsRestartNeeded      
  );  

~~~

Then for the actual animation the :func:`CPC_<DeviceType>_WidgetAnimation`
will check if the function :func:`CPC_<WidgetType>_WidgetBodyAnimation` exists 
and otherwise call :func:`CPC_<DeviceType>_WidgetBodyAnimation`. 

~~~c

    
    // Determine callback name
    string sFunction = "CPC_"+sWidgetType+"_WidgetBodyAnimation";
    if(!isFunctionDefined(sFunction) || sFunction == ""){
        sFunction = "CPC_VP_I_WidgetBodyAnimation";	
    } 
  
    // OBJECT ANIMATION (animation according to ObjectSt dpe)
    callFunction(sFunction, bOn, false, sBodyColor, fPosSt);

~~~

In these function the actual panel animation code is implemented. The arguments
to these function can differ per object, but as they are only for internal use
this doesn't matter. 

The functionality described above means we can implement both a default:

~~~c

    // Animation function for the default ion pump widget
    CPC_VP_I_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, float fPosSt){
        /* Function body */
    }

~~~

And an alternative animation function for different widgets:

~~~c
    
    // Animation function for the combined analog and pump widget
    CPC_VP_I_Analog_WidgetBodyAnimation(bool bOn, bool bHalf, string sBodyColor, float fPosSt){
        /* Function body */
    }

~~~

## Annotated Example

We have seen so far how we can determine the active widget letters and the body
color using the generic vacuum library. If all the above is merged together we 
can actually see what is happening. 




@page menu Contextual Menu

All the vacuum objects have an extended contextual menu which allows easy 
modification and rapid navigation.

\image html SCADA_Context_Menu.png
\image latex SCADA_Context_Menu.png

UNICOS defines a two methods for managing the contextual menu.
The function `CPC_<deviceType>_MenuConfiguration` configure the contextual menu and 
the function `CPC_<deviceType>_HandleMenu` handle actual click event from the menu

~~~c

.. function:: void CPC_<deviceType>_MenuConfiguration(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)

    Configures the menu by returning a dyn_string in the format that the WinCCOA
    function `popupMenu` can use. The resulting menu will be passed by
    reference in the dsMenuList paramater

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the popupMenu function

.. function:: CPC_<deviceType>_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
    
    This function handles the answer coming back from the popupMenu. Which means
    it will call the action related with the menu item selected 
    
    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsMenuList: The list of menu items as supplied in the function `CPC_<deviceType>_MenuConfiguration`
    :param int iMenuAnswer: The identifier of the selected menu item as supplied in the function `CPC_<deviceType>_MenuConfiguration`

~~~

To ease the handling of menus and to generate a coherent vacuum menu a few 
helper functions are defined in the generic ``unVacObject`` library. 
These functions can be used to extend the menu depending on the object.

Menu Configuration
------------------

For the configuration of the menu the following helper functions are defined:

~~~c

.. function:: uint unVacObject_addAliasActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string dsMenuList, [int iPosition = 0])
    
    Add a quick copy button for the alias

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the `popupMenu` function
    :param uint iPosition: The position at which the menu items are inserted, if no position is given they are appended to the end of the menu

    :return: The actual position on which the menu items are inserted

.. function:: uint unVacObject_addAnalogActionToMenu(string sDeviceName, string sDpType, mDpe, dyn_string dsAccessOk, dyn_string dsMenuList, [int iPosition = 0])
    
    Add the analog values to the menu

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the the function `popupMenu` function
    :param uint iPosition: The position at which the menu items are inserted, if no position is given they are appended to the end of the menu

    :return: The actual position on which the menu items are inserted

.. function:: uint unVacObject_addDisableEnableActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string dsMenuList, [int iPosition = 0])
    
    Add quick enable/disable functionality to the menu

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the `popupMenu` function
    :param uint iPosition: The position at which the menu items are inserted, if no position is given they are appended to the end of the menu

    :return: The actual position on which the menu items are inserted

.. function:: uint unVacObject_addVacuumExtensionActionToMenu(string sDeviceName, string sDpType, dyn_string dsAccessOk, dyn_string dsMenuList, [int iPosition = 0])
    
    Add the help, workorder creation and state/even history links to the menu

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the `popupMenu` function
    :param uint iPosition: The position at which the menu items are inserted, if no position is given they are appended to the end of the menu

    :return: The actual position on which the menu items are inserted

~~~

After feedback of the user it was decided to redesign the contextual menu of 
the vacuum objects. The goal was to reduce the length of the menu and to 
incorporate a custom vacuum trending tool, since the UNICOS trending proved
insufficient. This resulted in another helper function 
`unVacObject_addProcessActionToMenu` which rearranges the default UNICOS
menu items and stores them in sub menus.

~~~c

.. function:: uint unVacObject_addProcessActionToMenu(string sDeviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string dsMenuList, [int iPosition = 0])
        
    Add the unicos menu as a process and expert submenus

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param string dyn_string dsMenuConfig: The UNICOS configuration of the menu, possible values can be found in `cpcGenericObject_addUnicosActionToMenu` and the function `cpcGenericObject_addDefaultUnicosActionToMenu`
    :param dyn_string dsAccessOk: The access control
    :param dyn_string dsMenuList: Pop-up menu to show, dyn_string to be given to the popupMenu function
    :param uint iPosition: The position at which the menu items are inserted, if no position is given they are appended to the end of the menu

    :return: The actual position on which the menu items are inserted

~~~

If we then round everything up an example of a menu configuration for the `VP_TP`
would like the following:

~~~c


    CPC_VP_TP_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
        // Define the UNICOS menu configuration
        dyn_string dsMenuConfig = makeDynString(
            UN_POPUPMENU_SELECT_TEXT, 
            UN_POPUPMENU_ACK_TEXT, 
            UN_POPUPMENU_FACEPLATE_TEXT, 
            CPC_POPUPMENU_ALARMS_TEXT, 
            UN_POPUPMENU_TREND_TEXT, 
            CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT
        );

        // Define which analog values to display
        mapping mDpe;
        mDpe["Rotation"] = "PosSt";

        // Add device alias as copy button to the top of the menu
        unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
        // Add the analog values to the menu as disabled buttons
        unVacObject_addAnalogActionToMenu(deviceName, dpType, mDpe, dsAccessOk, menuList);

        // Add enable/disable buttons to the menu
        unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
        
        // Add the vacuum tools, like this online help, unVacStateHistory, unVacTrending, 
        // and links to create workorders and inspection in InforEAM
        int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
        
        // Add the UNICOS actions to the expert and process submenus
        iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
    }

~~~

## Menu answer handling

The handling is done the same as for the other UNICOS devices in 
the function `CPC_<DeviceType>_HandleMenu` function. 
The ``unVacObject`` library supplies a generic function for handling these 
menu actions the function `unVacObject_handleCustomUnicosMenu`

~~~c

.. function:: bool unVacObject_handleCustomUnicosMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer, string sHelpFile, dyn_string &dsExceptionInfo){
        
    Handle the answer of the popup menu for the vacuum actions. Will return false if the action is not found. 
    This is different than the normal UNICOS, but useful because the total context menu of UNICOS can be 
    overwritten

    :param string sDeviceName: Device DP name
    :param string sDpType: Device type
    :param dyn_string dsMenuList: The list of menu items as supplied in the function `CPC_<deviceType>_MenuConfiguration`
    :param int iMenuAnswer: The identifier of the selected menu item as supplied in the function `CPC_<deviceType>_MenuConfiguration`
    :param string sHelpFile: The help file is a link to an online help, which will most likely be a page in this manual. 
    :param dyn_string &dsExceptionInfo: The exception info supplied by action called in case of an error

    :return: Returns true if the action is found and executed, false if no action is executed

~~~

If we would then implement the function `CPC_<DeviceType>_HandleMenu` according for the ``VP_TP`` with the help of 
the function `unVacObject_handleCustomUnicosMenu` if would look like the following:

~~~c


    CPC_VP_TP_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
        dyn_string exceptionInfo;
        // Define the help file for this object. This should actually be in a global object constant/parameters
        string helpFile = "https://edms.cern.ch/file/1735644/1/Vacuum_Turbomolecular_Pump_Software_User_Manual.pdf";
        
        // First see if the action is found within the custom vacuum actions. If so do nothing more. 
        if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
          // If the the action is not found check whether it's a standard UNICOS action
          cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
        }
        // You might want to build in logic to show an error message when no action is matched with the menu answer
        
        if (dynlen(exceptionInfo) > 0) {
            unSendMessage_toExpertException("Right Click", exceptionInfo);
        }
    }

~~~





@page Faceplate

# Introduction

The faceplate is the property window of a widget. In this specific information 
about the state of the object can be found. See an example below of the VP_G
faceplate. 

\image html SCADA_VP_G_Faceplate.png
\image latex SCADA_VP_G_Faceplate.png

# Animation

To animate this panel UNICOS defines tree functions for every widget 
function `CPC_<DeviceType>_FaceplateInitStatics`, function `CPC_<DeviceType>_FaceplateDPEs`
and function `CPC_<DeviceType>_FaceplateStatusAnimationCB`. 

~~~c

.. function: void CPC_<DeviceType>_FaceplateInitStatics(string sDeviceName)
    
    This function is used to load any information which is static for the
    lifetime of the faceplate. This means it is mostly used for defining
    globals which are used in the actual animation
    
    :param: string sDeviceName: The name of the object
    
.. function:: void CPC_<DeviceType>_FaceplateDPEs(string sDeviceName, dyn_string &dsDpes) 
    
    In this function the DPE's needed to animate the faceplate are registred

    :param: string sDeviceName: The name of the object
    :param: dyn_string dsDpes: The list of DPE's to which the DPE's needed for the faceplate should be added to

.. function:: void CPC_<DeviceType>_FaceplateStatusAnimationCB(dyn_string dsDpes, dyn_anytype daValues)

    The animation callback function which is executed when one of the DPE's
    listed in function `CPC_<DeviceType>_FaceplateDPEs` changes.

    :param: dyn_string dsDpes: The list of DPE names registered in function `CPC_<DeviceType>_FaceplateDPEs`
    :param: dyn_string daValues: The list of current DPE values associated with the names

~~~

A very basic example is the animation for the `VR_AH` object.

~~~c
        
        CPC_VR_AH_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values){
            unVacObject_animateFaceplateObjectStatus(dpes, values);

            // Animates the on/off status and objectSt panel
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_POSST", true, "cpcColor_Faceplate_Status");

            // Animates operation modes, like auto, manual, forced and their inhibit modes
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUMOST", true, "cpcColor_Faceplate_Status");
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FOMOST", true, "cpcColor_Faceplate_Status");
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHFMO", true, "cpcColor_Faceplate_Status");

            cpcGenericObject_animateMaskEvent(dpes, values);
            cpcGenericObject_animateNeedRestart(dpes, values);

            // Animates warning modes
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FODIPRO", true, "cpcColor_Faceplate_Warning");
            unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MIOERBRST", true, "cpcColor_Faceplate_Warning");
        }

~~~

As you can see to help building the faceplate animation both the UNICOS, UCPC 
and unVacuum library supply the developer with a few helper functions.
A few external functions are only briefly described here. More can be found in the
UNICOS documentation and in the ``unFaceplate.ctl``, ``unGenericObject.ctl`` and in the 
`cpcGenericObject.ctl` libraries

- function `unGenericObject_ColorBoxAnimate`: Animate the checkbox on the faceplate
- function `unGenericObject_ColorBoxDisconnect`: Animate the disconnection of the checkbox on the faceplate
- function `unFaceplate_animateTranslatedValue`: Animate an integer value which should be mapped against a dictionary string
- function `unFaceplate_fetchAnimationCBOnlineValue`: Retrieve the value matching a DPE from the list values supplied to function `CPC_<DeviceType>_FaceplateStatusAnimationCB`
- function `unFaceplate_animateStatusBit`: Animate a checkbox according to the bit in the ``StsReg##``
- function `unFaceplate_animateInterlock`: Animate the interlock box according to the interlock DPE's
- function `cpcGenericObject_animateMaskEvent`: Animate the UCPC mask event status
- function `cpcGenericObject_animateNeedRestart`: Animate the UCPC need restart status

Then the most handy functions which the unVacuum component adds are:

~~~c

.. function:: void unVacObject_animateFaceplateOnlineValues(dyn_string dsDpes, dyn_anytype daValues, dyn_string dsDpeNames)
    
    This function automatically animates/updates the analog values of all the DPE's supplied in 
    ``dsDpeNames``. This includes automatic retrieval of the format and unit from the DPE itself and 
    finally animating the field with the same name as supplied in the ``dsDpeNames``
    
    :param dyn_string dsDpes: The list of DPE names registered in function `CPC_<DeviceType>_FaceplateDPEs`
    :param dyn_string daValues: The list of current DPE values associated with the names
    :param dyn_string dsDpeNames: The list of DPE names of which the analog value should be animated


.. function:: void unVacObject_animateFaceplateObjectStatus(dyn_string dsDpes, dyn_anytype daValues)

    This function automatically animates/updates the ObjectSt field, which every object in the unVacuum 
    component has. This includes automatic retrieval of the mapping and updating the field in the faceplate
    names ``ObjectSt``
    
    :param dyn_string dsDpes: The list of DPE names registered in function `CPC_<DeviceType>_FaceplateDPEs`
    :param dyn_string daValues: The list of current DPE values associated with the names

~~~

For an object with both an `ObjectSt` and some analog values, like the `VP_I`, this would look like this:

~~~c

    CPC_VP_I_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
        // Device online value animation. PostSt, APosSt, VPosSt
        unVacObject_animateFaceplateOnlineValues(dpes, values, makeDynString("PosSt", "APosSt", "VPosSt"));

        // Device status animation
        unVacObject_animateFaceplateObjectStatus(dpes, values);

        // Do other animation
        // ...
    }

~~~





@page state State History

## Introduction

State history functionality was created some years ago within Vacuum Control System framework. This is one of the features, widely used by vacuum control and not having direct equivalent in UNICOS framework. The implementation of state history was selected as one of first targets for the vacuum control convergence to UNICOS.

As one can guess from the name – the State History is a tool to display history of device states. One of main objectives was to display the history in a very compact form. In order to achieve this – the concept of ‘integral’ (or ‘main’) device state was introduced. If we consider, for example, sector valve – the overall state of valve includes many aspects: if valve is open/closed, are there interlocks, is valve controlled remotely or locally etc. However, the main purpose of the valve is to open or close corresponding vacuum pipe. Thus, we can define ‘main’ state to be rather simple: the valve is either open or closed. Of course, after some thinking some more states can be added to these 2 very basic states, for example: in error, no communication etc.. But, anyway, the set of ‘main’ states is very restricted and each state can be expressed as (relatively) short string.

It was decided that state history shall be represented as a table with rows, corresponding to the moments in time. Which times shall be displayed in table? Clearly – the moments when state was changed. This seems to be natural from user point of view, and this also well fits to archiving of DPE values in WinCC OA – most often they are archived ‘on change’. So, simple state history can look like this:

Table 1 Simple state history:

\image html simpleState.png
\image latex simpleState.png

When number of rows in state history is long enough – it will be much simpler to user if each state is assigned corresponding color, and color appears as background color of cell with state. With such display - visual recognition of important changes becomes easier:

Table 2 Simple state history with colours:

\image html stateColor.png
\image latex stateColor.png

It can be useful in many cases ##to see simultaneously state history for several devices – for example, to see how some process was evolving in time, by looking at state transitions of related devices. The decision for displaying state history of several devices was the following:

-	table displays all times when state of either device was changed
-	the state of each device is shown in own column
-	the particular cell (corresponding to given moment in time and given device) is filled with state/colour if device state was changed at this moment; the cell is left empty if device state was not changed at this moment.

Thus, resulting state history for several devices (two in this example) may look as following:

Table 3 State history for two devices:

\image html statemulticolor.png
\image latex statemulticolor.png

We called such presentation ‘chess board’.

One can see that state of Device2 at times Time2 and Time3 is the same on Table 3, so why two rows appear in the table? Because something has changed at moment Time3 for this device – the ‘main’ state remains ‘State1’, but something else has changed that does not affect ‘main’ state. If we consider valve example – the state was ‘open’ at Time2 and it is still ‘open’ at Time3, but something else has changed - for example, hardware interlock signal has appeared. From the chess board above we can see – something has changed, but we can’t see WHAT has changed if that change does not affect ‘main’ state.

Thus, more detailed information is needed… but still main presentation shall be kept compact. The decision was: user can select cell with state in the table (just by clicking on cell) – and more information will be shown beneath that chess board. If, for example, we decide that for particular device changes of StsReg01 and SteReg02 are essential for state history – all bits of StsReg01 and StsReg02 for selected moment in time + device will be shown beneath the chessboard.

It could be that at given moment (for example, Time3) only StsReg02 was changed, hence, we have the value of StsReg02 from archive. What shall be shown for StsReg01 at this moment? The decision looks evident: if only StsReg02 was changed at this moment – we display last value of StsReg01 before this moment: StsReg01 was not changed; hence, its value is still the same as it was before.

##	Design 

###	Inter-panel communication

In current vacuum framework popup menu for device contains item to display state history of this device. If state history is not open at this moment – the panel with state history will be opened and history for this device will be shown. If state history is already open – device is added to the list of devices, already shown in opened state history. The same approach is used for UNICOS-based implementation.

Such decision requires some kind of communication between two panels:

-	the panel where device popup menu was invoked
-	the state history panel

Current implementation of such communication uses auxiliary DP. The Figure below shows the communication with auxiliary DP:

\image html stateComDp.png
\image latex stateComDp.png

DP has the only DPE of type dyn_string, containing list of device names to be shown in history. State history panel just connects to that DPE when created and displays history for the list of devices according to DPE content.

In reality panel with device popup menu does not writes to DP directly, instead it calls corresponding library function – and it is that function who is responsible for:

-	Finding out appropriate communication DP. It is expected that at most one state history panel can be opened per UI. The name of DP just includes UI number. If DP for given UI number does not exist – it will be created
-	Adding device name to the list of names, already found in that DP
-	Writing resulting device name list to DP
–	Opening state history panel if it is not open yet

###	State history panel design

The state history panel widely uses new features, available in WinCC OA 3.15, namely:

-	layout: panels became ‘really resizable’
-	public properties/methods/events of panel: with this feature panel becomes real object (well, almost real) and resulting state history panel can be built of small independent blocks as illustrated by the following picture:
 
Figure 2 Panel design:

\image html stateDesign.png
\image latex stateDesign.png

The overall state history panel consists of two major parts:

-	ChessBoard, displaying table with timestamps and device states
-	Details, displaying details of status register(s) of selected device at selected time
Both parts are implemented as independent panels. The ChessBoard panel has event, which is triggered when user selects cell in the table. The Details panel has public method which can be called to display register details.

The details panel is in turn composed of two other building blocks (panels): one for displaying value of register where each bit has own meaning (like StsReg01), another one for displaying value of register where the whole value means something, i.e. the value is interpreted as enum (like ObjectSt).

Thus, the ‘state history’ panel itself has very little functionality in it:

-	manage geometry of children panels
-	receive cell selection event from ChessBoard panel and call public method of Details panel in order to display detailed information

###	Data query

Clearly, the state history shall read the history of DPE values from archive. The very first question: which DPEs are required to build state history? The answers are different for different device types.

Furthermore, later, when ‘main’ states shall be shown in the table, the similar question arises: how to convert value(s) of DPE(s) to string representation of ‘main’ state and to color, corresponding to that state? The answer is again device type specific.

The design of state history panel is: the panel itself handles generic algorithm to query and display data, it knows nothing about device type specific. Whenever some kind of specific information is needed – panel calls some library function which handles such data.

The code is organized in 2 major layers:

-	scripts/libs/vacuumExtension/vacuumExtension.ctl – contains ‘generic’ functions. Basically, each generic function just determines the type of device and calls type-specific function, performing real job
-	scripts/libs/vacuumObjects/<type>.ctl – implement type-specific functions for corresponding equipment type.

The following generic functions from the 1st library above are uses by state history:

Function name	Purpose

vacuumExtension_StateDPEs()	Build list of DPE names whose history shall be read in order to display state history for given device.

vacuumExtension_StateMainColorAndString()	Calculate string for ‘main state’ plus color based on values of all DPEs, returned by previous function

Table 4 Generic functions of CTL library

Function name |	Purpose
------------- | --------
vacuumExtension_StateDPEs() |	Build list of DPE names whose history shall be read in order to display state history for given device.
vacuumExtension_StateMainColorAndString() |	Calculate string for ‘main state’ plus color based on values of all DPEs, returned by previous function


The logic of panel for data query is rather simple – for each device to be shown in state history:

-	Call function vacuumExtension_StateDPEs() to get list of DPE names whose history shall be read
-	Read from archive history values for all these DPEs and pass the queried values to dedicated CTL DLL for further processing (see further in this document). The panel does not try to interpret queried values.

###	Processing of archived values

After querying of all archived values for all DPEs of all devices, to be shown in state history, is finished – these values shall be processed. The main tasks for processing are:

-	build ordered list of all timestamps for all devices/DPEs - this will define number of rows in chessboard
-	find values of all DPEs for given device at given moment in time – these values will be used to find ‘main’ state and color to be shown in cell. 
-	plus some more auxiliary tasks… the two tasks above are really the most important ones

In vacuum framework such ‘data processor’ was implemented as CTL DLL (the code is written in C++). The main reason for implementing this in C++ was performance: processing several (potentially) very long lists of archived data can be very slow in WinCC OA CTL language.

Finally, C++ code became so optimized (in terms of performance) that idea came: may be the same goal can now be achieved using code in WinCC OA CTL language? This step (migration C++ to CTL) was also planned as a ‘training’ to learn new features of WinCC OA 3.15. However, evaluation of CTL++ has finished with ‘CTL++ is not usable yet’ conclusion – see https://its.cern.ch/jira/browse/ENS-19044.

By this reason data processing engine was again implemented as CTL DLL, with code written in C++. In fact, this is slightly adopted version of similar CTL DLL, used in vacuum framework. The goal of adaptation was mainly to remove any references to Eqp class (and subclasses) such that C++ code only knows device names and DPE names, and it knows nothing about device functionality, meaning of DPE values etc.

Finally C++ code is just storage for DPE data read from archives plus a set of methods for search in this data set. C++ code does not try to interpret the values or device / DPE names in any way.

###	Display cell content in ChessBoard

####	Which rows to fill

Adding many empty rows to the table works very fast. However, it was found that calculation/display of cell contents in the table works rather slow: if table contains thousands rows and all of them to be updated at once – delay for user becomes unacceptable.

But in reality, even if table contains many rows – only limited number of rows is visible at a moment (typically – 10…30, depending on panel size). Setting cell values for tens rows is fast enough. So, it was decided: the cells are only filled for visible rows. This means when user scrolls very long table – the cells are filled on the fly, as they become visible. Still for very short time the user can see cells empty while scrolling the table, but that time is really short and there were no complains about such behavior so far.

Once cells are filled – the content is kept inside the table and we don’t need to refill cells for previously filled rows. But now the question comes: how to understand if cells for row were already filled or not? This can happen, for example, when user scrolls the table up and down and exposes to visible area the range of rows that was already exposed before. As one can see in Table 3 some cells in each row may be left empty on purpose, so checking current content of cell will not help – unless we check all cells in row. But this may take significant time if many devices are shown, i.e. if there are many columns in table.

Fortunately, every row contains one ‘cell’ which must be filled for each exposed row, namely: the very first cell with timestamp. In real panel timestamp is shown not in the cell, but rather in row header, but this difference does not matter for the logic.

So, the processing when some row(s) became visible as a result of scroll (or similar) operation is:

-	check if row header is empty or not
-	if row header is empty – make filling of row content, including row header and all cells of this row
-	if row header is already not empty – there is nothing to do for this row.

####	Which cells to fill

It was explained before (see Table 3) that some cells shall be empty, while some other shall have content (text + background color). What is the criterion? The decision is very simple:

for each cell in row the function of CTL DLL (see 2.1.4) is called with arguments: timestamp (corresponds to row number) and device name (corresponds to column number). If there is archived value for at least one DPE of this device at this moment – result will be not empty, hence, the state shall be calculated and shown in the cell. Otherwise result, returned by CTL DLL function, is empty and cell shall remain empty as well. It is very simple…

####	What to display in cell

To calculate string presentation of ‘main’ state and background color for cell – function vacuumExtension_StateMainColorAndString() is used (see Table 4). Again, this ‘generic’ function finds a type of device and calls type-specific function which does the real job.

The DPE values, passed as argument to this function, are found by corresponding function in CTL DLL for given time.

####	Display register details

It shall be easy to display details of register value provided that the code has information on:

-	meaning of every bit in register word (applicable to registers like StsReg0x)
-	meaning of every possible value of register (applicable to registers like ObjectSt)

Clearly, such information is device type specific. In vacuum framework such (similar) information is read from text configuration file when program starts and then data is kept in memory for further fast access.

For this implementation more simple decision was used: the data are still kept in ‘ready to use’ state in memory, but that memory is not filled as a result of parsing some file. Instead this ‘register info pool’ is filled by dedicated functions in CTL library. This information can also be read from type definition XML file, produced by TCT tool of UAB.

The main storage unit of information is CTL data type mapping. The storage is organized as following:
 
Figure 3 Register info organization:

\image html stateRegister.png
\image latex stateRegister.png

There is one ‘root’ mapping with keys = device type name. Each value in this mapping is another mapping – all data for given device type.

Device type mapping uses register names as keys (like StsReg01, ObjectSt…). Each value in this mapping is another mapping – all data for given register.

And the final step of this hierarchy is register mapping – it contains textual information to be shown in the panel. As it was mentioned before, all status registers are classified to two main groups depending on the way how the register value shall be interpreted and visualized:

1.	‘Bit type’ register – each bit of the whole value has independent meaning.
2.	‘Value type’ register – the whole value of register has special meaning (like enum in programming languages).

So, register mapping first of all contains pair:

~~~
key :string (“type”); value :int (0|1)
~~~

This is to determine how content of this mapping shall be interpreted.

And then register mapping contains textual information, to be shown in UI:

-	for ‘bit type’ register: 

~~~
key :int (bit #, 0…); value :dyn_string (1st = short name, 2nd = long name)
~~~

- for ‘value type’ register: 

~~~
key :uint (register value); value : string (value meaning)
~~~

The filling of this information is realized using approach, similar to the one used in other cases above – the device type mapping is filled by type-specific function if such function is defined.

##	Implementation details

### CTL DLL

The source code for CTL DLL can be found in repository https://gitlab.cern.ch/UCPCVac/unVacQtCode.

Single CTL DLL is used – unVacuum/UnVacHistoryData.dll (or .so for Linux) – which realizes data storage, processing and accessing logic, described in 2.1.4. The source code can be found in UnVacHistoryData directory in repository.

CTL language interface is realized by class UnVacHistoryData; one can see in the file UnVacHistoryData.cpp all functions of this DLL, available for WinCC OA CTL code.

Internally storage of values from archives is organized as following:
 
The below Figure show the storing of archive values in C++ classes

\image html stateArchive.png
\image latex stateArchive.png

All data for one state history panel are stored in (and accessible through) instance of `StateHistory` class. The instance is created by calling `UnVacStateArchiveCreate()` function from DLL; when state history panel is terminated – resources for data storage shall be released by calling `UnVacStateArchiveDelete()` function. The internal ID of `StateArchive` instance, returned by `UnVacStateArchiveCreate()`, shall be used by other calls.

One can wonder: what is the need for ID of instance – assuming there can be several instances – if by design only one state history panel can be opened by UI? The reason is simple: the same DLL for data/storage processing is also used by data replay facility.

Each instance of `StateHistory` contains list of `DpValuesSet` instances; the `DpValuesSet` class holds all data for all DPEs of one device.

Each instance of `DpValuesSet` contains list of `DpeValuesSet` instances; the `DpeValuesSet` class holds data for one DPE.

And finally – each `DpeValuesSet` contains list of `DpeValue` instances; the `DpeValue` is a smallest unit of data storage – it contains just one value and corresponding timestamp.

###	DP type

State history uses dedicated DP type for inter-panel communications (see 2.1.1). DP type name is `_VacStateHistory`. One DP is created (when needed) for every UI, the name of created DP is `_VacStateHistory_<ui_number>`.

###	WinCC OA CTL libraries

State history uses 3 CTL libraries, located in
 scripts/libs/vacuumExtension/

In principle functions in these libraries are short and simple enough, plus the code has comments, so no detailed information is provided here – just a mapping of functions to the design decisions, described above.

####	vacStateHistory.ctl

This library contains, first of all, set of functions to work with DP for inter-panel communications. These are the functions:

~~~
      vacStateHistory_getCtlDpName()
      vacStateHistory_addDdevice()
      vacStateHistory_removeDevice()
~~~

####	vacuumExtension.ctl

This library contains set of functions to find out history of which DPEs are required to build state history for given device (see 2.1.3). See also Table 4.

####	vacDpeBitInfo.ctl

This library contains set of functions to provide information for register detail display. There is only one public ‘general’ function `vacDpeBitInfo_getDeviceBitInfo()`

plus a set of device type specific functions which are called by this one: `_vacDpeBitInfo_initTypeBitInfo_<type_name>()`

####	Panels

The panels are located in:

~~~
     vision/vacPanels/stateHistory
~~~

Chess Board panel: `StateHistoryTable.pnl`

Details panel:	`RegisterInfo.pnl`

Bit info panel:	`RegisterBitInfo.pnl`

Enum info panel:	`RegisterValueInfo.pnl`

Parent panel, combining all panels above	`StateHistory.pnl`

The code in each of these panels is rather simple, it just implements the logic, described in this document.


