/**
  test_vacuumExtension.ctl contains all the code used for testing the vacuumExtension functionality
*/

#uses "vacuumExtension/vacuumExtension.ctl"
#uses "fwUnitTestComponentAsserts.ctl"
#uses "unVacObjects/cpcVP_TP.ctl"

// Artifical test device name
const string VACUUM_TEST_CPC_VP_TP_DEVICE_NAME = "test";
// The list of DPEs configured with the VP_TP for the color determination
const dyn_string VACUUM_TEST_CPC_VP_TP_DPES = makeDynString(
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".frontend.alarm", 
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".frontend.enabled",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.EvStsReg01",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.EvStsReg01:_online.._invalid",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.EvStsReg02",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.EvStsReg02:_online.._invalid",     
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.PosSt",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.PosSt:_online.._invalid",   
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.MotPosSt",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.MotPosSt:_online.._invalid",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.ObjectSt",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".ProcessInput.ObjectSt:_online.._invalid",
    VACUUM_TEST_CPC_VP_TP_DEVICE_NAME + ".disabled"
);

// Default values related with the DPE's for which the object is in valid state
const dyn_anytype VACUUM_TEST_CPC_VP_TP_VALUES = makeDynAnytype(
    false, true, 0, false, 0, false, 0, false, 0, false, 0, false, false
);
/**
  Tests the state dpe's dispatching and defaults values

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateDPEs(){
  dyn_string dsDpes;
  CPC_VP_TP_StateDPEs(VACUUM_TEST_CPC_VP_TP_DEVICE_NAME, dsDpes);
  dyn_string dsExp = unVacObject_dynSlice(VACUUM_TEST_CPC_VP_TP_DPES, 3);
  // Assert that the expected datapoints VACUUM_TEST_CPC_VP_TP_DPES (without the system integretity) is equal to the result
  assertEqual(dsExp, dsDpes);
}

/** 
  Tests if the non valid animation is correct and if it has the highest priority.
  This is done by testing the different valid dpes with all the bits high of the status register.
  And by verifying it's not selected when all the validness values are false
*/
void test_CPC_VP_TP_StateMainColorAndString_NotValid(){
  string sColor, sState; // The result color  and state string

  // The list of dpe names  
  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES;
  // The keys in the dpe value array(daValues) that correspond to validness
  dyn_int diKeys = makeDynInt(4, 6, 8, 10, 12);
  // All bits on, worst case scenario
  bit32 bit32StsReg = 4294967295;
  
  // The mocked dpe values
  dyn_anytype daValues = VACUUM_TEST_CPC_VP_TP_VALUES;   
  // All the bits high to check the highest priority of unvalid
  daValues[3] = daValues[5] = bit32StsReg; 
  // Test when everything is valid
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unDataNotValid" == sColor);
  assertFalse(CPC_STATE_INVALID == sState);
  
  // Loop over all the validness key positions in daValues and check them all seperately  
  for(int i = 1; i<=dynlen(diKeys); i++){
    daValues[diKeys[i]] = true; // Set invalid to true
    CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
    daValues[diKeys[i]] = false; // Reset dpe mocked values
    
    // Assert invalid color 
    assertEqual("unDataNotValid", sColor);
    assertEqual(CPC_STATE_INVALID, sState);
  }
}


/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateMainColorAndString_AlarmUnack(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values
  
  // The status register with CPC_StsReg01_ALUNACK high 
  bit32 bit32StsReg01;
  setBit(bit32StsReg01, CPC_StsReg01_ALUNACK, true);
  
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;  //RESET 
  // Test with all bits high 
  daValues[3] = daValues[5] = 4294967295; // All bits high
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
 
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES; //RESET
  // Test with only CPC_StsReg01_ALUNACK bit high 
  daValues[3] = bit32StsReg01;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
  
  // Test with no bits high 
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;   
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unWidget_AlarmNotAck" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES; //RESET
  
  // Test with UNICOS acknowledge mechanism
  daValues[3] = bit32StsReg01;
  dynAppend(daValues, 0); //iStartIStState
  dynAppend(daValues, 1); //iTStopIStState
  dynAppend(daValues, 1); //iFuStopIStState 
  dynAppend(daValues, 0); //iAlSt
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState, false, true);
  assertEqual("unWidget_AlarmNotAck", sColor);
  
  // Test with UNICOS acknowledge mechanism without CPC_StsReg01_ALUNACK bit
  daValues[3] = 0;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState, false, true);
  assertFalse("unWidget_AlarmNotAck" == sColor);
}

/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateMainColorAndString_Alarm(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values

  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03;

  // No interlock active  
  // Build dpe values
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("unFaceplate_AlarmActive" == sColor);
  assertFalse(CPC_STATE_ALARMUNACK == sState);
  
  // CPC_StsReg01_TSTOPIST active   
  setBit(bit32StsReg01, CPC_StsReg01_TSTOPIST, true);  // Set bit
  // Build dpe values
  daValues[3] = bit32StsReg01;
  // Test function
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);     
  assertEqual("unFaceplate_AlarmActive", sColor);  
  assertEqual(CPC_STATE_ALARM, sState);
  setBit(bit32StsReg01, CPC_StsReg01_TSTOPIST, false);  // Unset bit

  // CPC_StsReg01_TSTOPIST active   
  setBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST, true);  // Set bit
  // Build dpe values  
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;
  daValues[5] = bit32StsReg02;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unFaceplate_AlarmActive", sColor);
  assertEqual(CPC_STATE_ALARM, sState);
  setBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST, false);  // set bit
  
   // CPC_StsReg01_ALST active   
  setBit(bit32StsReg01, CPC_StsReg01_ALST, true);  // Unset the first local interlock bit
  // Build dpe values
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;  
  daValues[3] = bit32StsReg01;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);   
    
  assertEqual("unFaceplate_AlarmActive", sColor);
  assertEqual(CPC_STATE_ALARM, sState);
  setBit(bit32StsReg01, CPC_StsReg01_ALST, false);  // Unset the first local interlock bit
}

/**
  Test the disabled color of the widget/state
  
  @covers vacuumExtension_StateMainColor
*/
void test_CPC_VP_TP_StateMainColorAndString_Disabled(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values  

  // No interlock active  
  // Build dpe values
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertFalse("FwEquipmentDisabled" == sColor);
  assertFalse(CPC_STATE_DISABLED == sState);  
  
  daValues[dynlen(daValues)] = true;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);
  assertEqual("FwEquipmentDisabled", sColor);
  assertEqual(CPC_STATE_DISABLED, sState);  
}

/**
  Tests the state main color for warnings

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateMainColorAndString_Warning(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues; // The corresponding DPE values    
  
  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03; // Status registers
  
  // Test valve in OK status
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES; // RESET
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("cpcColor_Widget_Warning" == sColor);
  assertFalse(CPC_STATE_WARNING == sState);  
  
  // Test explicit restart needed shows WARNING
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState, true, false, true);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  
  // Test valve in CPC_StsReg01_IOERRORW status
  setBit(bit32StsReg01, CPC_StsReg01_IOERRORW, true);
  daValues[3] = bit32StsReg01;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("cpcColor_Widget_Warning", sColor);
  assertEqual(CPC_STATE_WARNING, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_IOERRORW, false);
}

/**
  Tests the state main color for alarms

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateMainColorAndString_Forced(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues = VACUUM_TEST_CPC_VP_TP_VALUES; // The corresponding default DPE values    
  
  bit32 bit32StsReg01, bit32StsReg02, bit32StsReg03; // Status registers
  
  // Test valve in OK status
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("unWidget_ControlStateForced" == sColor);
  assertFalse(CPC_STATE_FORCED == sState);    
    
  // Test valve in CPC_StsReg01_FoMoSt status
  setBit(bit32StsReg01, CPC_StsReg01_FoMoSt, true);
  daValues[3] = bit32StsReg01;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unWidget_ControlStateForced", sColor);
  assertEqual(CPC_STATE_FORCED, sState);  
  setBit(bit32StsReg01, CPC_StsReg01_FoMoSt, false);
  daValues = VACUUM_TEST_CPC_VP_TP_VALUES; //RESET
  
  // Test valve in transition state, i.e. Accelerating, Deccelerating, Recovering status
  dyn_int diObjectSt = makeDynInt(1, 3, 4);
  for(int i = 1;i<=dynlen(diObjectSt); i++){
    daValues[11] = diObjectSt[i];
    CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
    assertEqual("unWidget_ControlStateForced", sColor);
    assertEqual(CPC_STATE_UNDEFINED, sState);  
  }
}

/**
  Tests the state main color for off state in the state history

  @covers vacuumExtension_StateMainColor  
*/
void test_CPC_VP_TP_StateMainColorAndString_OffState(){
  string sColor, sState; // The result color  and state string

  dyn_string dsDpes = VACUUM_TEST_CPC_VP_TP_DPES; // List of DPE names
  dyn_anytype daValues = VACUUM_TEST_CPC_VP_TP_VALUES; ; // The corresponding DPE values    
    
  // Test valve in ON status
  daValues[13] = 2;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertFalse("unEventList_ForeColor" == sColor);  
  assertFalse(CPC_STATE_OFF == sState);  
  
  // Test valve in OK status as widget
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState, false);  
  assertFalse("unEventList_ForeColor" == sColor);
  assertFalse(CPC_STATE_OFF == sState);  

  // Test valve in OFF status as widget
  daValues[13] = 0;
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState, false);  
  assertFalse("unEventList_ForeColor" == sColor);   
  assertFalse(CPC_STATE_OFF == sState);  
  
    // Test valve in OFF status in state history
  CPC_VP_TP_StateMainColorAndString(dsDpes, daValues, sColor, sState);  
  assertEqual("unEventList_ForeColor", sColor);
  assertEqual(CPC_STATE_OFF, sState);  
}
