const string UNVACUUM_TEST_DEVICE_NAME = "vacuum_test";
const string UNVACUUM_TEST_DEVICE_TYPE = "Vacuum_TestObject";

global mapping UNVACUUM_TEST_CALL_COUNTER;

/** 
  Mock function for the StateDPEs
  
  @param sDeviceName 
  @param dsDpes 
*/
void Vacuum_TestObject_StateDPEs(string sDeviceName, dyn_string &dsDpes){
  assertEqual(UNVACUUM_TEST_DEVICE_NAME, sDeviceName);
  assertEqual(2, dynlen(dsDpes), "Failed to assert length "+dynlen(dsDpes)+" is 2");
  incrementCallCounter("Vacuum_TestObject_StateDPEs");
  
  // Register dpe
  dynAppend(dsDpes, UNVACUUM_TEST_DEVICE_NAME+".ProcessInput.PosSt");
}

/**
  Mock function for the StateMainColor

  @param dsDpes 
  @param daValues 
*/
string Vacuum_TestObject_StateMainColorAndString(dyn_string dsDpes, dyn_anytype daValues, string &sColor, string &sState){
  dyn_string dsDpesExp = makeDynString("frontend.alarm", "frontend.enabled", UNVACUUM_TEST_DEVICE_NAME+".ProcessInput.PosSt");  
  dyn_anytype daValuesExp = makeDynAnytype(true, true, 1);
  assertEqual(dsDpesExp, dsDpes);
  assertEqual(daValuesExp, daValues);
  incrementCallCounter("Vacuum_TestObject_StateMainColorAndString");
  
  sColor = "crap";
  sState = "more crap";  
  
  return sColor;
}

/**
  Mock function for the StateMainColor

  @param dsDpes 
  @param daValues 
*/
string Vacuum_TestObject_StateErrorMessage(dyn_string dsDpes, dyn_anytype daValues){
  dyn_string dsDpesExp = makeDynString("frontend.alarm", "frontend.enabled", UNVACUUM_TEST_DEVICE_NAME+".ProcessInput.PosSt");  
  dyn_anytype daValuesExp = makeDynAnytype(true, true, 1);
  assertEqual(dsDpesExp, dsDpes);
  assertEqual(daValuesExp, daValues);
  incrementCallCounter("Vacuum_TestObject_StateErrorMessage");
  
  return "crap";
}

/**
  Mock function for the StateMainColor

  @param dsDpes 
  @param daValues 
*/
string Vacuum_TestObject_StateWarningMessage(dyn_string dsDpes, dyn_anytype daValues){
  dyn_string dsDpesExp = makeDynString("frontend.alarm", "frontend.enabled", UNVACUUM_TEST_DEVICE_NAME+".ProcessInput.PosSt");  
  dyn_anytype daValuesExp = makeDynAnytype(true, true, 1);
  assertEqual(dsDpesExp, dsDpes);
  assertEqual(daValuesExp, daValues);
  incrementCallCounter("Vacuum_TestObject_StateWarningMessage");
  
  return "crap";  
}
/** 
  Increment the call counter for tracking function calls
  
  @param sFunction The function name
*/
private void incrementCallCounter(string sFunction){
  if(!mappingHasKey(UNVACUUM_TEST_CALL_COUNTER, sFunction)){
    UNVACUUM_TEST_CALL_COUNTER[sFunction] = 0;
  }
  UNVACUUM_TEST_CALL_COUNTER[sFunction] = UNVACUUM_TEST_CALL_COUNTER[sFunction] + 1;    
}

/**
  Assert that the function is actually called
  
  @param string sFunctionName The name of the function
  @param any_type The expected arguments of the function
*/
assertFunctionCall(string sFunctionName, int iTimes = 1){
  if(iTimes <= 0) return;
  assertTrue(mappingHasKey(UNVACUUM_TEST_CALL_COUNTER, sFunctionName), "Function "+sFunctionName+" is expected to be called "+iTimes+" times, actually called 0 times");
  assertTrue(UNVACUUM_TEST_CALL_COUNTER[sFunctionName] == iTimes, "Function "+sFunctionName+" is expected to be called "+iTimes+" times, actually called "+UNVACUUM_TEST_CALL_COUNTER[sFunctionName]+" times");
  UNVACUUM_TEST_CALL_COUNTER[sFunctionName] = 0;
}
