/**
Function for vacuum instruments with vacuum hierarchy

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, systemName/dpType
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_VAC(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    dyn_string dsParameters, dsSplit;
    string sLocation, sApplication, sDomaine, sNature;
   
    sHierarchy = cs_unLHCLoggingDBUserLib_Error;
    sDescription = "Cannot retrieve logging config !";
    
    sDpeExtension = sFullDpeName;

    string deviceName = unGenericDpFunctions_getDpName(sFullDpeName);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    string sApplicationName; 
    int ret= dpGet("_unApplication.applicationName", sApplicationName);
    dsSplit = strsplit(sApplicationName, " ");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = sApplicationName;
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
        int i = 1;
        sDomaine = "";
        while (sDomaine == "") {
            sDomaine = dsT1[i];
            i++;
        }
    } else {
        sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    }
    if (sDomaine == PROJ) {
        sDomaine = sLocation;
    }      

    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, ".");
    int iLen = dynlen(dsSplit);
    if (iLen == 3) {	
        // UNICOS Device
        if (dsSplit[2] == "ProcessInput") {            
            
            sDescription = unGenericDpFunctions_getDescription(deviceName);
            sDpAlias = dpGetAlias(dsSplit[1] + ".");
            sDpeExtension = substr(sFullDpeName, strpos(sFullDpeName, ".") + 1, strlen(sFullDpeName));
            sName = sDpAlias + "." + dsSplit[3];

            if (sLocation == "") {
                DebugN("ERROR: Location is empty for " + deviceName);
                return;
            }
            if (sApplication == "") {
                DebugN("ERROR: Application is empty for " + deviceName);
                return;
            }
            if (sNature == "") {
                DebugN("ERROR: Nature is empty for " + deviceName);
                return;
            }
            if (sDomaine == "") {
                DebugN("ERROR: Domain is empty for " + deviceName);
                return;
            }         
                       
            if (patternMatch("*SPARE*", strtoupper(sDescription))) {
                sHierarchy = cs_unLHCLoggingDBUserLib_Ignore;
            } else {
                sHierarchy = "VAC/" + sLocation + "/" + sDomaine + "/"+ sNature;    //VAC hierarchy
            }
            
        } else {
            DebugN("ERROR: 'ProcessInput' pattern not found !");
        }
    } else if (iLen == 2) {		
        // System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sDpeExtension, sDescription, sDpAlias, sName, sHierarchy);
    } else { 
        // nothing to do
    }
}
