/**

This library contains the import and export function of the CPC_VG_F.

@date 30/10/2017

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
	Thijs van Winden (TE-VSC)
	Sebastien Blanchard (TE-VSC)

@copyright
        &copy;Copyright CERN 2017 - all rights reserved
*/

const float CPC_VG_F_PROTECTION_MAX = 1.3e3;
const float CPC_VG_F_PROTECTION_MIN = 1e-11;
    


// Constants
const string UN_CONFIG_CPC_VG_F_DPT_NAME = "CPC_VG_F";
const unsigned UN_CONFIG_CPC_VG_F_LENGTH                     = 32;
//begin_TAG_SCRIPT_DEVICE_TYPE_constants
const unsigned UN_CONFIG_CPC_VG_F_UNIT                       = 1;
const unsigned UN_CONFIG_CPC_VG_F_FORMAT                     = 2;
const unsigned UN_CONFIG_CPC_VG_F_HHLIMIT                    = 3;
const unsigned UN_CONFIG_CPC_VG_F_HLIMIT                     = 4;
const unsigned UN_CONFIG_CPC_VG_F_LLIMIT                     = 5;
const unsigned UN_CONFIG_CPC_VG_F_LLLIMIT                    = 6;
const unsigned UN_CONFIG_CPC_VG_F_ALARM_ACTIVE               = 7;
const unsigned UN_CONFIG_CPC_VG_F_DEADBAND                   = 8;
const unsigned UN_CONFIG_CPC_VG_F_DEADBAND_TYPE              = 9;
const unsigned UN_CONFIG_CPC_VG_F_ARCHIVE_ACTIVE             = 10;
const unsigned UN_CONFIG_CPC_VG_F_ARCHIVE_TIME_FILTER        = 11;
const unsigned UN_CONFIG_CPC_VG_F_OBJECTST_ARCHIVE_ACTIVE    = 12;
const unsigned UN_CONFIG_CPC_VG_F_OBJECTST_ARCHIVE_TIME_FILTER = 13;
const unsigned UN_CONFIG_CPC_VG_F_ACK_ALARM                  = 14;
const unsigned UN_CONFIG_CPC_VG_F_NORMAL_POSITION            = 15;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_STSREG01           = 16;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_STSREG02           = 17;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_EVSTSREG01         = 18;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_EVSTSREG02         = 19;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_POSST              = 20;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_HFST               = 21;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_OBJECTST           = 22;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_MPOSRST            = 23;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_GPRHST           = 24;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_GPRLST           = 25;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_MANREG01           = 26;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_MPOSR              = 27;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_MGPRHR             = 28;
const unsigned UN_CONFIG_CPC_VG_F_ADDRESS_MGPRLR             = 29;
const unsigned UN_CONFIG_CPC_VG_F_DGPRH_DEFAULT              = 30;
const unsigned UN_CONFIG_CPC_VG_F_DGPRL_DEFAULT              = 31;
const unsigned UN_CONFIG_CPC_VG_F_MAPPING                    = 32;

const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_LENGTH          = 8;

const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_DISABLED        = 2;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_PARAMETERS      = 3;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_MASTER_NAME     = 4;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_PARENTS         = 5;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_CHILDREN        = 6;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_TYPE            = 7;
const unsigned UN_CONFIG_CPC_VG_F_ADDITIONAL_SECOND_ALIAS    = 8;

//end_TAG_SCRIPT_DEVICE_TYPE_constants



/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_VG_FConfig_getConfig() {
    mapping config;
    mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg01";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg01"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg02";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg02"] 		= props;

    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg01";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg01"] 	= props;

    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg02";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg02"] 	= props;

    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    props["hasSmooth"]      = true;
    props["hasArchive"]     = true;
    props["5RangesAlerts"]  = true;
    props["address"]        = ".ProcessInput.PosSt";
    props["dataType"]       = CPC_FLOAT;
    config["PosSt"]         = props;

    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    // props["hasSmooth"]      = true;
    props["address"]        = ".ProcessInput.HFSt";
    props["dataType"]       = CPC_FLOAT;
    config["HFSt"]          = props;
    
    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    // props["hasSmooth"]      = true;
    props["address"]        = ".ProcessInput.MPosRSt";
    props["dataType"]       = CPC_FLOAT;
    config["MPosRSt"]          = props;
    
    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    // props["hasSmooth"]      = true;
    props["address"]        = ".ProcessInput.GPrHSt";
    props["dataType"]       = CPC_FLOAT;
    config["GPrHSt"]          = props;
    
    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    // props["hasSmooth"]      = true;
    props["address"]        = ".ProcessInput.GPrLSt";
    props["dataType"]       = CPC_FLOAT;
    config["GPrLSt"]          = props;
    
    mappingClear(props);
    props["hasArchive"]     = true;
    props["hasMapping"]     = true;
    props["address"]        = ".ProcessInput.ObjectSt";
    props["dataType"]       = CPC_UINT16;
    config["ObjectSt"]        = props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.StartISt";
    props["dpe"]			= "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_STARTIST;
    props["isAlarm"]		= true;
    config["StartISt"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.TStopISt";
    props["dpe"]			= "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_TSTOPIST;
    props["isAlarm"]		= true;
    config["TStopISt"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.FuStopISt";
    props["dpe"]			= "StsReg02";
    props["bitPosition"]	= CPC_StsReg02_FUSTOPIST;
    props["isAlarm"]		= true;
    config["FuStopISt"] 	= props;

    mappingClear(props);
    props["address"]      = ".ProcessInput.AlSt";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALST;
    props["isAlarm"]      = true;
    config["AlSt"]        = props;

    mappingClear(props);
    props["address"]      = ".ProcessInput.AlUnAck";
    props["dpe"]          = "StsReg01";
    props["bitPosition"]  = CPC_StsReg01_ALUNACK;
    config["AlUnAck"]     = props;

    mappingClear(props);
    props["address"] 		= ".ProcessOutput.ManReg01";
    props["dataType"] 		= CPC_UINT16;
    config["ManReg01"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessOutput.MPosR";
    props["dataType"] 		= CPC_FLOAT;
    config["MPosR"] 		= props;
    
    mappingClear(props);
    props["address"] 		= ".ProcessOutput.MGPrHR";
    props["dataType"] 		= CPC_FLOAT;
    config["MGPrHR"] 		= props;    
    
    mappingClear(props);
    props["address"] 		= ".ProcessOutput.MGPrLR";
    props["dataType"] 		= CPC_FLOAT;
    config["MGPrLR"] 		= props;
//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

    return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_VG_FConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
    return makeDynString(CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PARAMS_HALFFULLLIMIT);
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/** Check custom configuration
*/
CPC_VG_FConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig    
    float fValue;
    
    // Check the default values
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VG_F_DGPRH_DEFAULT], fValue, exceptionInfo);
    unConfigGenericFunctions_checkFloat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VG_F_DGPRL_DEFAULT], fValue, exceptionInfo);  
    
  
    int shift = UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_VG_F_LENGTH;
    if(hasArchive){
        shift += UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
    }
    
    bool bBool;
    cpcConfigGenericFunctions_checkBool("DISABLED", configLine[shift + UN_CONFIG_CPC_VG_F_ADDITIONAL_DISABLED], bBool, exceptionInfo);

//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
*/
CPC_VG_FConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
  
    dyn_string exceptionInfo;
    
    /// Set the defaults
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DGPrH", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VG_F_DGPRH_DEFAULT]);
    dpSetWait(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.DGPrL", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VG_F_DGPRL_DEFAULT]);
    
    // Set the ranges
    fwPvRange_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.GPrHSt",              
              CPC_VG_F_PROTECTION_MIN,
              CPC_VG_F_PROTECTION_MAX,
              UN_CONFIG_DEFAULT_RANGE_NEGATE_RANGE,
              UN_CONFIG_DEFAULT_RANGE_IGNORE_OUTSIDE,
              UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MIN,
              UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MAX,
              exceptionInfo);
    
    fwPvRange_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.GPrLSt",
              CPC_VG_F_PROTECTION_MIN,
              CPC_VG_F_PROTECTION_MAX,
              UN_CONFIG_DEFAULT_RANGE_NEGATE_RANGE,
              UN_CONFIG_DEFAULT_RANGE_IGNORE_OUTSIDE,
              UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MIN,
              UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MAX,
              exceptionInfo);

    // Attach the acknowledgment propagation
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.TStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.FuStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.StartISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.AlSt");
    
    
    // Add the disabled functionality from the spec sheet
    int shift = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VG_F_LENGTH;
    if(hasArchive){
        shift += UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
    }   
    
    dpSet(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".disabled", dsConfigs[shift + UN_CONFIG_CPC_VG_F_ADDITIONAL_DISABLED]);
    

    if(dsConfigs[shift + UN_CONFIG_CPC_VG_F_ADDITIONAL_DISABLED]){
        dpSet(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".eventMask", true);
    }
}

/**
Purpose: Export CPC_VG_F Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_VG_FConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    string sObject = UN_CONFIG_CPC_VG_F_DPT_NAME;
    int amount = dynlen(dsDpList);
    for (int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if (!dpExists(currentDP)) { continue; }

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//begin_TAG_SCRIPT_DEVICE_TYPE_export
        
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_get5AlertLimits(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getAlarmActive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters, ".ProcessInput.ObjectSt");
        cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(currentDP, ".ProcessInput.PosSt", dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.StartISt", ".ProcessInput.TStopISt", ".ProcessInput.FuStopISt"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01",  true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02",  true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt",       true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "HFSt",        true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosRSt",     true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "GPrHSt",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "GPrLSt",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt",    true, dsDpParameters);
        
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01",    false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR",       false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MGPrHR",      false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MGPrLR",      false, dsDpParameters);

        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "DGPrH");
        cpcExportGenericFunctions_getDefaultValue(currentDP, dsDpParameters, "DGPrL");        

        dynAppend(dsDpParameters, cpcGenericDpFunctions_getMapping(currentDP, ".ProcessInput.ObjectSt"));
        
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt", "ProcessInput.ObjectSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);

        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);

        string disabled;
        dpGet(currentDP + ".disabled", disabled);
        dynAppend(dsDpParameters, disabled);
        
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_VG_F Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_VG_F_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VG_FConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_VG_F Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_VG_F_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VG_FConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VG_F_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VG_FConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VG_F_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VG_FConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
