/**@file

// cpcVG_PT.ctl
This library contains the widget, faceplate, etc. functions of VG_PT.

@par Creation Date
  dd/mm/yyyy

@par Modification History

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Marcin Bes (BE-ICS)
  Sebastien Blanchard (TE-VSC)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{


/** Returns the list of VG_PT DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VG_PT_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VG_PT_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    dpGet(deviceName + ".ProcessInput.PosSt", value,
          deviceName + ".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VG_PT_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
    dyn_string exceptionInfo;
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT, CPC_POPUPMENU_ALLOW_TO_RESTART_TEXT);

    // List of analog values displayed int he context menu
    mapping mDpe;
    mDpe["Pressure"] = "PosSt";

    string sControllerName = unVacuum_VG_PT_getControllerName(deviceName);

    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    unVacObject_addAnalogActionToMenu(deviceName, dpType, mDpe, dsAccessOk, menuList);

    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    unVacObject_addItemToPopupMenu(menuList, "Controller", CPC_POPUPMENU_CONTROLLER, true);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);

//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** gets name of the controller - first object on "parents" list

@param deviceName input, device DP name
@return result string
*/
string unVacuum_VG_PT_getControllerName(string deviceName){
  dyn_string parents;
  dyn_string exceptionInfo;
  unGenericDpFunctions_getKeyDeviceConfiguration(deviceName, CPC_CONFIG_PARENTS_KEY, parents, exceptionInfo);
  if(dynlen(parents) > 0) {
    return parents[1];
  }
  else {
    return "";
  }
}

/** handle the answer of the popup menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VG_PT_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
//begin_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
    dyn_string exceptionInfo;

    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);
    // The name of the controller
    string controllerName = unVacuum_VG_PT_getControllerName(deviceName);

    // Handle the specific to this device
    switch(menuAnswer){
      case CPC_POPUPMENU_CONTROLLER:
        if(controllerName != ""){
          string sDpName = unGenericDpFunctions_getWidgetDpName(unGenericDpFunctions_getSystemName(deviceName) + controllerName);
          unVacObject_OpenFaceplate(sDpName, makeDynString("$sRefDpName:"+deviceName), exceptionInfo);
        }
      break;
      // Use default vacuum handling. If not found use default UNICOS handling
      default:
      if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
        cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
      }
    }

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
    //end_TAG_SCRIPT_DEVICE_TYPE_HandleMenu
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VG_PT_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
    int iAlertType1, iAlertType2, iAlertType3, iAlertType4;
    dyn_string exceptionInfo;
    g_bMRestart     = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    //g_bURValid      = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_URVALID, exceptionInfo);
    //g_fHalFullLimit = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HALFFULLLIMIT, exceptionInfo);
    g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");

    int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                     deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                     deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                     deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    cpcGenericObject_GetAnalogAlarmType(deviceName + ".ProcessInput.PosSt", g_iAlertType, exceptionInfo);

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of VG_PT DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VG_PT_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs

    dynAppend(dpes, deviceName + ".eventMask");                                // 6
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                    // 7
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 8
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                    // 9
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 10
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                    // 11
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 12
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");                       // 13
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");     // 14
    dynAppend(dpes, deviceName + ".disabled");                                 // 15

    if(g_bBoolAlert) {
      dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 16
      dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 17
      dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 18
      dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 19
    }
    if (g_iAlertType){
      dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state");  // 20 or 16
    }

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VG_PT_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

  // Do nothing if system is not connected
	if (!g_bSystemConnected){
		return;
	}
  int iSystemIntegrityAlarmValue = values[1];
  bool bSystemIntegrityAlarmEnabled = values[2];
  bool bLocked = values[3];
  string sSelectedManager = values[4];
  bool bComment = values[5];

  int iMask = values[6];
  bit32 bit32StsReg01 = values[7];
  bit32 bit32StsReg02 = values[9];
  int iObjectSt = values[11];
  float fPosSt = values[13];
  bool bDisabled = values[15];

  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt, iHState = 0;
  if (g_bBoolAlert) {
      iStartIStState = values[15];
      iTStopIStState = values[16];
      iFuStopIStState = values[17];
      iAlSt = values[18];
      if(g_iAlertType) {
        iHState = values[19];
      }
  } else if (g_iAlertType) {
     iHState = values[15];
  }

   // Check all invalid dpes for invalidness.
  bool bInvalid = values[8] || values[10] || values[12] || values[14];

  // Create list the status registers
  dyn_bit32 dbit32StsRegs = makeDynBit32(bit32StsReg01, bit32StsReg02);

  // Determine widget type
  bool bCompact = widgetType == "VG_PT";

  // Base letter bits set on the widget type
  dyn_string dsWarningBits = (bCompact) ? VAC_WARNING_BITS : CPC_WARNING_BITS;
  dyn_string dsAlarmBits = (bCompact) ? VAC_ALARM_BITS : CPC_ALARM_BITS;
  dyn_string dsControlBits = (bCompact) ? VAC_CONTROL_BITS : CPC_CONTROL_BITS;

  // Use default warning letter to list
  dyn_bool dbWarningBools = unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsWarningBits);

  // Use default alarm letters
  dyn_bool dbAlarmBools = unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsAlarmBits);     // Default vacuum alarm bits
  // Check whether the device needs a restart
  bool bIsRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_StsReg02_NEEDRESTART) == 0;
  dbAlarmBools[5] = bIsRestartNeeded;

  // Generic vacuum letter animation
  unVacObject_WidgetLetterAnimation(widgetType, UN_CONFIG_CPC_VG_PT_DPT_NAME,
			bLocked, sSelectedManager, dbit32StsRegs,
      dbWarningBools,    // Default vacuum warning bits
      CPC_WARNING_LETTERS, // Default vacuum warning letters
      dbAlarmBools,
      CPC_ALARM_LETTERS, // Default vacuum alarm letters
      unVacObject_fetchBitsFromStsRegs(dbit32StsRegs, dsControlBits) ,    // Default vacuum control bits
      CPC_CONTROL_LETTERS, // Default vacuum control letters
			bInvalid, bIsRestartNeeded, bComment,
			iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);

  // Determine the body color based on the state
  dyn_int diKeyFilter = makeDynInt(3, 4, 5, 13, 14);

  // Filter dpes based on index (remove dpe not used for color or string animations)
  string sBodyColor, sState;
  dyn_string filteredDpes = unVacObject_dynWithoutIndex(dpes, diKeyFilter);
  dyn_anytype filteredValues = unVacObject_dynWithoutIndex(values, diKeyFilter);
  CPC_VG_PT_ColorAndString("", filteredDpes, filteredValues, sBodyColor, sState, false, g_bBoolAlert, bIsRestartNeeded);

  // Determine callback name
	string sFunction = "CPC_"+widgetType+"_WidgetBodyAnimation";
	if(!isFunctionDefined(sFunction) || sFunction == ""){
		sFunction = "CPC_VG_PT_WidgetBodyAnimation";
	}

  // OBJECT ANIMATION (animation according to ObjectSt dpe)
  callFunction(sFunction, sBodyColor, iObjectSt, fPosSt);
  //end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/**
  Widget body animation for gauge "happy faces" widget

  @param sBodyColor The body color determined by the state
  @param iObjectSt The object state number
*/
void CPC_VG_PT_WidgetBodyAnimation(string sBodyColor, int iObjectSt, float fPosSt) {
  int iBodyFill = 0; // 0 - empty, 1 - halffull, 2 - full
  switch(iObjectSt) {
    case 2: // if Gauge On
    case 3: // if Gauge is Under Range
      iBodyFill = 2;
      break;
    case 8:  // High Pressure
      iBodyFill = 1;
      break;
  }
  setMultiValue("Body1",            "foreCol",  sBodyColor,

                "Body2",            "foreCol",  sBodyColor,
                "Body2",            "backCol",  sBodyColor,

                "Body3",            "foreCol",  sBodyColor,
                "Body3",            "backCol",  sBodyColor,

                "Body4",            "foreCol",  iBodyFill >= 2 ? "unWidget_Background" : sBodyColor,

                "Body5",            "foreCol",  iBodyFill >= 1 ? "unWidget_Background" : sBodyColor,
                "Body6",            "foreCol",  iBodyFill >= 1 ? "unWidget_Background" : sBodyColor,

                "Body2",            "visible",  iBodyFill >= 2,
                "Body3",            "visible",  iBodyFill >= 1,
                "WidgetArea",       "visible",  true
              );
  // Only animate PosSt if it exists
  if(!shapeExists("PosSt")) return;
  // DebugTN("fPosSt:",fPosSt);
  CPC_VG_PT_Analog_WidgetBodyAnimation(sBodyColor, iObjectSt, fPosSt);

}


/**
  Widget body animation for analog widget

  @param sBodyColor The body color determined by the state
  @param iObjectSt The object state number
*/
void CPC_VG_PT_Analog_WidgetBodyAnimation(string sBodyColor, int iObjectSt, float fPosSt) {
  string sPosSt;

  // Set display as off when the gauge is off
  switch(iObjectSt){
    case 2:
    case 3:
    case 8:
      sPosSt = unGenericObject_FormatValue(CPC_VAC_ANALOG_FORMAT, fPosSt); // + " " + g_sPosStUnit;
      break;
    default:
      sPosSt = "Off";
      break;
  }

  setMultiValue(
      "PosSt",            "foreCol",  sBodyColor,
      "PosSt",            "text",  sPosSt,
      "WidgetArea",       "visible",  true
  );
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VG_PT_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    string sFunction = "CPC_"+sWidgetType+"_WidgetBodyAnimation";
  	if(!isFunctionDefined(sFunction) || sFunction == ""){
  		sFunction = "CPC_VG_PT_WidgetBodyAnimation";
  	}

    // OBJECT ANIMATION (animation according to ObjectSt dpe)
    callFunction(sFunction, "unDataNoAccess", 0, 0.0);

    setMultiValue("WarningText",      "text",     "",
                  "AlarmText",        "text",     "",
                  "ControlStateText", "text",     "",
                  "SelectArea",       "foreCol",  "",
                  "LockBmp",          "visible",  false,
                  "WidgetArea",       "visible",  false,
                  "eventState",       "visible",  false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VG_PT_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    dyn_string exceptionInfo;

    buttons[UN_FACEPLATE_BUTTON_SELECT] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_AUTO_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_MANUAL_MODE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_FORCED_MODE] = UN_ACCESS_RIGHTS_EXPERT;

    buttons[UN_FACEPLATE_BUTTON_SET_VALUE] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_ONOPEN_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_ONFORCED_REQUEST] = UN_ACCESS_RIGHTS_OPERATOR;

    buttons[UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS] = UN_ACCESS_RIGHTS_EXPERT;

    buttons[UN_FACEPLATE_BUTTON_ACK_ALARM] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[CPC_FACEPLATE_BUTTON_ALLOW_RESTART] = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_MASKEVENT] = UN_ACCESS_RIGHTS_EXPERT;
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VG_PT_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VG_PT_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool bSelected, buttonEnabled, bit1, bit2, bit3;
    string localManager;
    dyn_string exceptionInfo;

    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    mixed stsReg01Value = dsData[3];
    bool bStsReg01Bad = dsData[4];

    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        switch (dsButtons[i]) {
            case UN_FACEPLATE_BUTTON_FORCED_MODE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_FOMOST) == 0);
                bit2 = !(getBit(stsReg01Value, CPC_StsReg01_AUINHFMO) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
            case UN_FACEPLATE_BUTTON_ONFORCED_REQUEST:
            case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
            case UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS:
            case UN_FACEPLATE_BUTTON_AUTO_MODE:
            case UN_FACEPLATE_BUTTON_SET_VALUE:
                bit1 = !(getBit(stsReg01Value, CPC_StsReg01_AUMOST) == 0);
                buttonEnabled = buttonEnabled && bSelected && !(bit1) && !bStsReg01Bad;
                break;
            case UN_FACEPLATE_BUTTON_MANUAL_MODE:
                buttonEnabled = buttonEnabled && bSelected &&
                                !getBit(stsReg01Value, CPC_StsReg01_MMOST) &&
                                !getBit(stsReg01Value, CPC_StsReg01_AUINHMMO) &&
                                !bStsReg01Bad;
                break;

            default:
                break;
        }
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        } else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VG_PT_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    int iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
    dyn_string exceptionInfo;
    int iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                     deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                     deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                     deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_params["BoolAlert"] = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    g_params["PosStFormat"] = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_params["PosStUnit"] = dpGetUnit(deviceName + ".ProcessInput.PosSt");
    g_params["HFStFormat"] = dpGetFormat(deviceName + ".ProcessInput.HFSt");
    g_params["HFStUnit"] = dpGetUnit(deviceName + ".ProcessInput.HFSt");
    g_params["MRestart"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    g_params["URValid"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_URVALID, exceptionInfo);
    g_params["HalfFullLimit"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HALFFULLLIMIT, exceptionInfo);

    g_params["ControllerName"] = unVacuum_VG_PT_getControllerName(deviceName);

    mapping objectStLabels;
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), objectStLabels);
    g_params["ObjectStLabels"] = objectStLabels;

    int iAlertType;
    cpcGenericObject_GetAnalogAlarmType(deviceName + ".ProcessInput.PosSt", iAlertType, exceptionInfo);
    g_params["iAlertType"] = (iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL);

    g_params["rangeClasses"] = makeDynString();

    if (g_params["iAlertType"]) {
        int rangesNumber;
        dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._num_ranges", rangesNumber);
        for (int i = 1; i <= rangesNumber; i++) {
            string rangeClass = "#";
            dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl." + i + "._class", rangeClass);
            dynAppend(g_params["rangeClasses"], rangeClass);
        }
    }

    //g_params["GaugeType"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_VG_PT_GAUGE_TYPE, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}

/** Returns the list of VG_PT DPEs which should be connected on faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VG_PT_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.HFSt");
    dynAppend(dpes, deviceName + ".ProcessInput.HFSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");

    if (g_params["BoolAlert"]) {
        dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state");
        dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");

    }
    if (g_params["iAlertType"]){

      mappingRemove(g_params, "alertPositions_H");

      dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._active");
      dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range");
      dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible");

      for (int i = 1; i <= dynlen(g_params["rangeClasses"]); i++) {
            string rangeClass = g_params["rangeClasses"][i];
            if (strpos(rangeClass, "_cpcAnalogH") > 0) {
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl." + i + "._act_state");
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl." + i + "._l_limit");
                g_params["alertPositions_H"] = i;
            }
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VG_PT_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
    unFaceplate_animateOnlineValue(dpes, values, "PosSt", g_params["PosStUnit"], g_params["PosStFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["ObjectStLabels"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTextParameter(dpes, values, "URValid", g_params["URValid"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTextParameter(dpes, values, "HalfFullLimit", unGenericObject_FormatValue(g_params["PosStFormat"], g_params["HalfFullLimit"]), "cpcColor_Faceplate_Status");

    unFaceplate_animateOnlineValue(dpes, values, "HFSt", g_params["HFStUnit"], g_params["HFStFormat"], "cpcColor_Faceplate_Status");

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUMOST",   true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MMOST",    true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHMMO", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FOMOST",   true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUINHFMO", true, "cpcColor_Faceplate_Status");

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_AUTOOV",   true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_SWOFFOV",  true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_SWONOV",   true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUOFFPRST",true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUONFRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_AUONPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MOFFPRST", true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MONFRST",  true, "cpcColor_Faceplate_Status");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_MONPRST",  true, "cpcColor_Faceplate_Status");

    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_STARTIST", "StartISt", "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_TSTOPIST", "TStopISt", "CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg02_FUSTOPIST","FuStopISt","CPC_StsReg01_ALUNACK");
    unFaceplate_animateInterlock(dpes, values, "CPC_StsReg01_ALST",     "AlSt",     "CPC_StsReg01_ALUNACK");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_ALUNACK",  true,       "cpcColor_Alarm_Bad");

    cpcGenericObject_animateMaskEvent(dpes, values);
    cpcGenericObject_animateNeedRestart(dpes, values);

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_FODIPRO", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_MIOERBRST", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg02_ALBW", true, "cpcColor_Faceplate_Warning");

    if (g_params["iAlertType"] == 0) { // if does not exist
      cpcGenericObject_ColorBoxAnimateWarning("Alarm_H", 0, false, false, false);
    } else {
      bool alarmActive = unFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl.._active");
      float alarmRange;
      int alarmState;
      if (mappingHasKey(g_params, "alertPositions_H")) {
        alarmRange = unFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_params["alertPositions_H"] + "._l_limit");
        alarmState = unFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_params["alertPositions_H"] + "._act_state");
        unGenericObject_DisplayValue(g_params["PosStFormat"], g_params["PosStUnit"], alarmRange, "ValueH", "unDisplayValue_Parameter", false);
        cpcGenericObject_ColorBoxAnimateWarning("Alarm_H", alarmState, g_params["iAlertType"], alarmActive, false);
      }
    }

    // controller button text
    if(g_params["ControllerName"] != "")
    {
      setValue("ControllerBtn", "toolTipText", g_params["ControllerName"]);
    } else {
      setValue("ControllerBtn", "enabled", "false");
    }



    // unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_POSALW", true, "cpcColor_Faceplate_Warning");
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

//@}

/**
  Lists the dpes required for building the state of the device

  @param string sDeviceName The name of the device which is in UNICOS the same as the dpname
  @param dyn_string dsDpes The list of datapoints elements required for building the state
  @param bool bForHistory The DPEs will be used for state history, i.e. only DPEs with _archive
                            config are of interest

*/

CPC_VG_PT_StateDPEs(string sDeviceName, dyn_string &dsDpes, bool bForHistory = true) {
    dynAppend(dsDpes, sDeviceName + ".eventMask");                                  // 3
    if(bForHistory) {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01");                    // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg01:_online.._invalid");  // 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02");                    // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.evStsReg02:_online.._invalid");  // 7
    }
    else {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01");                    // 4
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 5
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02");                    // 6
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 7
    }
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt");                      // 8
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.ObjectSt:_online.._invalid");    // 9
    dynAppend(dsDpes, sDeviceName + ".disabled");                                   // 10
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.PosSt");                         // 11
    dynAppend(dsDpes, sDeviceName + ".ProcessInput.PosSt:_online.._invalid");       // 12

    if(g_bBoolAlert) {
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 13
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 14
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 15
      dynAppend(dsDpes, sDeviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 16
    }
}

/**
  Returns the main color associated with the device state. This color is based on
  the dpes listed in the CPC_VG_PT_StateDPEs for the dedicated device type. In case of the
  state history the EvStsReg## is used because this is archived, while for determining
  the current state the StsReg## is used.
  @param string sDeviceName, imput, device dp name to get objectSt meaning
  @param dyn_string dsDpes The list of dpe names used to determine the color
  @param dyn_anytype daValues The list of dpe values used to determine the color

  Not filtered
      dynAppend(dpes, deviceName + ".eventMask");                                // 6
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");                    // 7
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");  // 8
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02");                    // 9
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg02:_online.._invalid");  // 10
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");                    // 11
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");  // 12
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt");                       // 13
    dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_online.._invalid");     // 14
    dynAppend(dpes, deviceName + ".disabled");                                 // 15

    if(g_bBoolAlert) {
      dynAppend(dpes, deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state");  // 16
      dynAppend(dpes, deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state");  // 17
      dynAppend(dpes, deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state"); // 18
      dynAppend(dpes, deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state");      // 19
    }
    if (g_iAlertType){
      dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state");  // 20 or 16



   dyn_ are filtered (3, 4, 5, 13, 14)

  1  int 		    frontend.alarm
  2  bool 		  frontend.enabled
  3  bool       device.eventMask                                     was 6
  4  bit32		  device.ProcessInput.(Ev)StsReg01
  5  bool 		  device.ProcessInput.(Ev)StsReg01:_online.._invalid
  6  bit32		  device.ProcessInput.(Ev)StsReg02
  7  bool		    device.ProcessInput.(Ev)StsReg02:_online.._invalid
  8  int		    device.ProcessInput.ObjectSt
  9  bool		    device.ProcessInput.ObjectSt:_online.._invalid
  10 bool		    device.disabled
  11 int		    device.ProcessInput.StartISt:_alert_hdl.._act_state
  12 int		    device.ProcessInput.TStopISt:_alert_hdl.._act_state
  13 int		    device.ProcessInput.FuStopISt:_alert_hdl.._act_state
  14 int		    device.ProcessInput.AlSt:_alert_hdl.._act_state

  @param string sColor, output, the main color string of device
  @param string sString, output, the main state string of device
  @param bool bUseOffColor, input, Off color is return in &sColor when device is Off, false in case of WidgetColor
  @param bool bBoolAlert, input, realtime alert according to UNICOS alert manager

  @param bool bIsRestartNeeded, input, there is a restart needed according to UNICOS device manager

  @see For the stsReg bit meaning check the specifications file: https://svn.cern.ch/reps/en-ice-svn/trunk/applications/vacuum/UNICOS_VAC_DEVICES/Design/Object_ManReg_StsReg_Mapping.xls
  @see Other usufull documentation: https://edms.cern.ch/ui/file/1228441/1.6.0/UCPC6_RP1.6.0_Manual.pdf

  @return 'o' when function proceed successfully, '-1' when function proceed with error
*/

int CPC_VG_PT_ColorAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, string &sState, bool bUseOffColor, bool bBoolAlert = false, bool bIsRestartNeeded = false){

  // --- Get device data value
  // --------------------------
  // Read system integrity values
  int iSystemIntegrityAlarmValue = daValues[1];
  bool bSystemIntegrityAlarmEnabled = daValues[2];
  bit32 bit32StsReg01 = daValues[4];
  bit32 bit32StsReg02 = daValues[6];
  // Object status
  int iObjectSt = daValues[8];
  bool bDisabled = daValues[10];
  // Software Interlock status
  int iStartIStState, iTStopIStState, iFuStopIStState, iAlSt;
  if (bBoolAlert) {
    iStartIStState = daValues[11];
    iTStopIStState = daValues[12];
    iFuStopIStState = daValues[13];
    iAlSt = daValues[14];}
  // Status valid (At least one of the DPE is invalid)
  bool bInvalid = daValues[5] || daValues[7] || daValues[9];

  // --- Get main state string
  //--------------------------
  if (sDeviceName!="") {
     sState = unVacObject_getStateString(sDeviceName, iObjectSt);
  }
  else {
     sState = "Unknown";
  }

  // --- Get main color string
  // --------------------------
  // Device status for color determination
  bool bUnderRange = iObjectSt == 3;
  bool bWarning = bIsRestartNeeded || getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1;
  bool bAlarmUnack = getBit(bit32StsReg01, CPC_StsReg01_ALUNACK) == 1 &&
             (iTStopIStState  == 1 ||
              iTStopIStState  == 3 ||
              iFuStopIStState == 1 ||
              iFuStopIStState == 3);
  bool bAlarm  = getBit(bit32StsReg01, CPC_StsReg01_TSTOPIST) == 1 || getBit(bit32StsReg02, CPC_StsReg02_FUSTOPIST) == 1 || getBit(bit32StsReg01, CPC_StsReg01_ALST) == 1;
  bool bForcedMode = getBit(bit32StsReg01, CPC_StsReg01_FoMoSt) == 1;
  bool bOff = iObjectSt == 0; // Off without error
  // Create boolen array for corresponding colors
  dyn_bool dbStateBool = makeDynBool(
      bInvalid,              // Invalid color
      bAlarmUnack,           // Alarm Unacknowledged color
      bAlarm,                // Alarm active color
      bDisabled,             // Device disabled color
      bWarning,              // Warning color
      bForcedMode,           // Forced mode color
      false,		             // Undefined color
      bOff && bUseOffColor,  // Off color (optional color)
      bUnderRange,           // Under Range color
      true                   // OK color
  );
  // Fetch the body color from CPC_BODY_COLORS based on the lowest index value of the array dbBodyColorBool is true
  unVacObject_WidgetBoolMapping(dbStateBool, CPC_BODY_COLORS, sColor);

  return 0;
}

int CPC_VG_PT_ColorFillAndString(string sDeviceName, dyn_string dsDpes, dyn_anytype daValues, string &sColor, bool &bFill, string &sState) {
  bFill = false;
  int coco = CPC_VG_PT_ColorAndString(sDeviceName, dsDpes, daValues, sColor, sState, true);
  if(coco >= 0) {
    int iObjectSt = daValues[8];
    switch(iObjectSt) {
      case 2: // if Gauge On
      case 3: // if Gauge is Under Range
      case 8:  // High Pressure
        bFill = true;
        break;
    }
  }
  return coco;
}
