/**@file

// cpcVP_TPConfig.ctl
This library contains the import and export function of the CPC_VP_TP.

@par Creation Date
  24/05/2016

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Sebastien Blanchard (TE-VSC)

@copyright
        &copy;Copyright CERN 2016 - all rights reserved
*/
//@{



// Constants
const string UN_CONFIG_CPC_VP_TP_DPT_NAME = "CPC_VP_TP";
//begin_TAG_SCRIPT_DEVICE_TYPE_constants

// import row: import row config line description : data value separated by ';'
//-------------
// // 10 GENERIC (CPC_VP_TP;deviceNumber;Alias[,DeviceLinkList];Description;Diagnostics;WWWLink;Synoptic;Domain;Nature;WidgetType;) 
// // 4(of 22) MAIN (MotPosStUnit;MotPosStFormat;ArchiveMode;TimeFilter;) 
// // 9(of 22) MAIN ADDRESS (addr_StsReg01;addr_StsReg02;addr_EvStsReg01;addr_EvStsReg02;addr_PosSt;addr_MotPosSt;addr_ManReg01;addr_ObjectSt;addr_ErrCoSt;)
// // 4(of 22) MAIN LIMIT (posStHHLimit;posStHLimit;posStLLimit;posStLLLimit;)
// // 6(of 22) MAIN ALARM and MAPPING (AlarmActive;SMSCat;AlarmMessage;AlarmAck;mapping_ObjectSt;)
// // 3 ARCHIVE (BooleanArchive;AnalogArchive;EventArchive;)
// // 10 ADDITIONAL (MaskEvent;Parameters;MasterDevice;parents;children;type;secondAlias;ControllerModel;TMPModel;mapping_ErrCoSt;)


const unsigned UN_CONFIG_CPC_VP_TP_POSST_UNIT	=	1	; // unit of PosSt
const unsigned UN_CONFIG_CPC_VP_TP_POSST_FORMAT	=	2	; // format of PosSt (ex: ##.#)
const unsigned UN_CONFIG_CPC_VP_TP_MOTPOSST_UNIT	=	3	; // unit of motPosSt
const unsigned UN_CONFIG_CPC_VP_TP_MOTPOSST_FORMAT	=	4	; // format of motPosSt (ex: ##.#)
const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_ACTIVE	=	5	; // up to three comma-separated values <archiveMode>, optional<archiveDeadband>, optional<archiveDeabandVsTimeLogic>   c.f.(1)
const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_TIME_FILTER	=	6	; // posSt and motPosSt archive time smoothing in second (ex: 180.0)
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_STSREG01	=	7	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_STSREG02	=	8	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_EVSTSREG01	=	9	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_EVSTSREG02	=	10	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_POSST	=	11	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_MOTPOSST	=	12	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_MANREG01	=	13	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_OBJECTST	=	14	;
const unsigned UN_CONFIG_CPC_VP_TP_ADDRESS_ERRCOST	=	15	;
const unsigned UN_CONFIG_CPC_VP_TP_HHLIMIT            	=	16	;
const unsigned UN_CONFIG_CPC_VP_TP_HLIMIT             	=	17	;
const unsigned UN_CONFIG_CPC_VP_TP_LLIMIT             	=	18	;
const unsigned UN_CONFIG_CPC_VP_TP_LLLIMIT            	=	19	;
const unsigned UN_CONFIG_CPC_VP_TP_ALARM_ACTIVE       	=	20	;
const unsigned UN_CONFIG_CPC_VP_TP_SMS_CATEGORIES     	=	21	;
const unsigned UN_CONFIG_CPC_VP_TP_SMS_MESSAGE        	=	22	;
const unsigned UN_CONFIG_CPC_VP_TP_ACK_ALARM    	=	23	;
const unsigned UN_CONFIG_CPC_VP_TP_MAPPING	=	24	; // Mapping of the meaning for ObjectSt

const unsigned UN_CONFIG_CPC_VP_TP_LENGTH                   = 24;

const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_BOOLEAN_ARCHIVE	= 1;
const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_ANALOG_ARCHIVE   = 2;
const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_EVENT_ARCHIVE    = 3;

const unsigned UN_CONFIG_CPC_VP_TP_ARCHIVE_LENGTH           = 3;

const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_MASKEVENT     = 1;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_PARAMETERS    = 2;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_MASTER_NAME   = 3;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_PARENTS       = 4;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_CHILDREN      = 5;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_TYPE          = 6;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_SECOND_ALIAS  = 7;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_CONTROLLER_MODEL= 8;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_INSTRUMENT_MODEL= 9;
const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_MAPPING_ERRCOST= 10;

const unsigned UN_CONFIG_CPC_VP_TP_ADDITIONAL_LENGTH        = 10;
//end_TAG_SCRIPT_DEVICE_TYPE_constants

//------------------------------------------------------
// NOTES :
// (1) ARCHIVE_ACTIVE
//�N� for NO archiving
//�VA,0.1,N� for Absolute Value archiving with 0.1 deadband and NO time
//�VR,0.1,N� as above, but Relative Deadband
//�Y� for Time-only archiving
//�VA,0.1,A� for Absolute Value archiving with 0.1 deadband AND Time
//�VR,0.1,A� for Relative Value archiving with 0.1 deadband AND Time  DEFAULT ARCHIVE SETUP FOR VACUUM ANALOG VALUE
//�VA,0.1,O� for Absolute Value archiving with 0.1 deadband OR Time
//�VR,0.1,O� for Relative Value archiving with 0.1 deadband OR Time
//�A� for Old/new comparison AND time
//�O� for Old/new comparison OR time



/**
DPE configuration
TODO: use fw info instead
*/
mapping CPC_VP_TPConfig_getConfig() {
    mapping config;
    mapping props;

//begin_TAG_SCRIPT_DEVICE_TYPE_deviceConfig
// 9 ADDRESS (addr_StsReg01;addr_StsReg02;addr_EvStsReg01;addr_EvStsReg02;addr_PosSt;addr_MotPosSt;addr_ManReg01;addr_ObjectSt;addr_ErrCoSt;)
// 1
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg01";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg01"] 		= props;
// 2
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StsReg02";
    props["dataType"] 		= CPC_UINT16;
    config["StsReg02"] 		= props;
// 3
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg01";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg01"] 	= props;
// 4
    mappingClear(props);
    props["hasArchive"] 	= true;
    props["address"] 		= ".ProcessInput.evStsReg02";
    props["dataType"] 		= CPC_INT32;
    config["evStsReg02"] 	= props;
// 5
    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    //props["hasSmooth"]      = false;
    props["5RangesAlerts"]  = true;
    props["address"]        = ".ProcessInput.PosSt";
    props["dataType"]       = CPC_FLOAT;
    config["PosSt"]         = props;
// 6
    mappingClear(props);
    props["hasUnit"]        = true;
    props["hasFormat"]      = true;
    //props["hasSmooth"]   	= false;
    props["address"]        = ".ProcessInput.MotPosSt";
    props["dataType"]       = CPC_FLOAT;
    config["MotPosSt"]      = props;
// 7
	mappingClear(props);
	props["hasMapping"]     = true;
    props["hasUnit"]        = false;
    props["hasFormat"]      = false;
    //props["hasSmooth"]   	= false; does not work
	//props["hasArchive"]     = true; archive setting done in custom function
    props["address"]        = ".ProcessInput.ObjectSt";
    props["dataType"]       = CPC_UINT16;
    config["ObjectSt"]      = props;
	// 9
    mappingClear(props);
	props["hasUnit"]        = false;
	//props["hasArchive"]     = true; archive setting done in custom function
    //props["hasSmooth"]   	= false; does not work
	props["hasArchive"]     = true;
    props["address"] 		= ".ProcessInput.ErrCoSt";
    props["dataType"] 		= CPC_UINT16;
    config["ErrCoSt"] 		= props;	
	
    mappingClear(props);
    props["address"] 		= ".ProcessInput.StartISt";
    props["dpe"]			= "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_STARTIST;
    props["isAlarm"]		= true;
    config["StartISt"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.TStopISt";
    props["dpe"]			= "StsReg01";
    props["bitPosition"]	= CPC_StsReg01_TSTOPIST;
    props["isAlarm"]		= true;
    config["TStopISt"] 		= props;

    mappingClear(props);
    props["address"] 		= ".ProcessInput.FuStopISt";
    props["dpe"]			= "StsReg02";
    props["bitPosition"]	= CPC_StsReg02_FUSTOPIST;
    props["isAlarm"]		= true;
    config["FuStopISt"] 	= props;

    mappingClear(props);
    props["address"]      	= ".ProcessInput.AlSt";
    props["dpe"]          	= "StsReg01";
    props["bitPosition"]  	= CPC_StsReg01_ALST;
    props["isAlarm"]      	= true;
    config["AlSt"]        	= props;

    mappingClear(props);
    props["address"]      	= ".ProcessInput.AlUnAck";
    props["dpe"]          	= "StsReg01";
    props["bitPosition"]  	= CPC_StsReg01_ALUNACK;
    config["AlUnAck"]     	= props;
// 8
    mappingClear(props);
    props["address"] 		= ".ProcessOutput.ManReg01";
    props["dataType"] 		= CPC_UINT16;
    config["ManReg01"] 		= props;


//end_TAG_SCRIPT_DEVICE_TYPE_deviceConfig

    return config;
}

/**
Returns the list of parameter names that passed via config line.
*/
dyn_string CPC_VP_TPConfig_getParamNames() {
//begin_TAG_SCRIPT_DEVICE_TYPE_getParamNames
    // TODO add gauge?
    return makeDynString(CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PROP_NAME_INTERFACE_TYPE);
//end_TAG_SCRIPT_DEVICE_TYPE_getParamNames
}

/** Check custom configuration
*/
CPC_VP_TPConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
//end_TAG_SCRIPT_DEVICE_TYPE_checkCustomConfig
}

/** Set custom configuration
*/
CPC_VP_TPConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
//begin_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.TStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.FuStopISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.StartISt");
    cpcDpFunc_fixAckPropogation(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], ".ProcessInput.AlSt");
	
	//Import the properties using cpcGenericDpFunctions_setDeviceProperty(string sDeviceDpName, string propertyName, string propertyValue, dyn_string &exceptionInfo)
	//Import Property CONTROLLER MODEL
	int iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_TP_ARCHIVE_LENGTH + UN_CONFIG_CPC_VP_TP_LENGTH + UN_CONFIG_CPC_VP_TP_ADDITIONAL_CONTROLLER_MODEL;
	cpcGenericDpFunctions_setDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_PROP_NAME_CONTROLLER_MODEL, dsConfigs[iConfigDataIndex], exceptionInfo);
	//Import Property INSTRUMENT MODEL	
	iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_TP_ARCHIVE_LENGTH + UN_CONFIG_CPC_VP_TP_LENGTH + UN_CONFIG_CPC_VP_TP_ADDITIONAL_INSTRUMENT_MODEL;
	cpcGenericDpFunctions_setDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_PROP_NAME_INSTRUMENT_MODEL, dsConfigs[iConfigDataIndex], exceptionInfo);
	//Import Description ErrCoSt Mapping
	//unGenericDpFunctions_setKeyDeviceConfiguration(sDeviceDpName, UN_CPC_DEVICE_PROPERTY_PREFIX + propertyName, makeDynString(propertyValue), exceptionInfo);
	iConfigDataIndex = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_VP_TP_ARCHIVE_LENGTH + UN_CONFIG_CPC_VP_TP_LENGTH + UN_CONFIG_CPC_VP_TP_ADDITIONAL_MAPPING_ERRCOST;
	unGenericDpFunctions_setKeyDeviceConfiguration(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], "DESC_ErrCoSt", dsConfigs[iConfigDataIndex], exceptionInfo);

	//Archive setting for ObjectSt and ErrCoSt
	//fwArchive_set(string dpe, string archiveClassName, int archiveType, int smoothProcedure, float deadband, float timeInterval, dyn_string &exceptionInfo, bool checkClass = TRUE)
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ObjectSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
	fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.ErrCoSt",
                  dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                  DPATTR_ARCH_PROC_SIMPLESM,
                  DPATTR_COMPARE_OLD_NEW,
                  0,
                  0,
                  exceptionInfo);
  
  int iInterfaceType = (int) cpcGenericDpFunctions_getDeviceProperty(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], CPC_PROP_NAME_INTERFACE_TYPE, exceptionInfo);
  
  // Don't create archives for PosSt and MotPosSt when using I/O interface

  if(iInterfaceType == 4 || iInterfaceType == 5) return;
  
  // Manual archive setting for PosSt and MotPosSt based on the InterfaceType of the VP_TP. I/O interface doesn't supply this data, thus archiving needs to be disabled 
  mapping mConfig = CPC_VP_TPConfig_getConfig();

  dyn_string dsDpes = makeDynString("PosSt", "MotPosSt"); // The dpe to be archived
  string sDeviceType = dsConfigs[UN_CONFIG_OBJECTGENERAL_OBJECT];
  string sPlcType = cpcConfigGenericFunctions_getFrontEndType(dsConfigs);
  
  // Loop over analog archive values
  for (int i = 1; i<=dynlen(dsDpes); i++){
    cpcConfigGenericFunctions_setArchive(dsConfigs, sDeviceType, dsDpes[i], mConfig[dsDpes[i]], sPlcType, hasArchive, exceptionInfo);
  }
   

	
//end_TAG_SCRIPT_DEVICE_TYPE_setCustomConfig
}

/**
Purpose: Export CPC_VP_TP Devices

Usage: External function

PVSS manager usage: NG, NV
*/
CPC_VP_TPConfig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    string sObject = UN_CONFIG_CPC_VP_TP_DPT_NAME;
    int amount = dynlen(dsDpList);
    for (int i = 1; i <= amount; i++) {
        dyn_string dsDpParameters;
        string currentDP = dsDpList[i];

        if (!dpExists(currentDP)) { continue; }

        unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//begin_TAG_SCRIPT_DEVICE_TYPE_export
        string sLimitOn, sLimitOff;
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters, ".ProcessInput.MotPosSt");
        cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters, ".ProcessInput.MotPosSt");
        cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.MotPosSt");
          
        dpGet(currentDP + ".ProcessOutput.PLiOn",  sLimitOn);
        dynAppend(dsDpParameters, unExportDevice_floatFormat(sLimitOn));
        dpGet(currentDP + ".ProcessOutput.PLiOff",  sLimitOff);
        dynAppend(dsDpParameters, unExportDevice_floatFormat(sLimitOff));
        cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.StartISt", ".ProcessInput.TStopISt", ".ProcessInput.FuStopISt"), dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", 		true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MotPosSt",      true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ObjectSt",      true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ErrCoSt",      true, dsDpParameters);
//        cpcExportGenericFunctions_processAddress(currentDP, sObject, "AuPosRSt",    true, dsDpParameters);
//        cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosRSt",     true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
//       cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR", 		false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PLiOn", 		false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(currentDP, sObject, "PLiOff", 		false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.OnSt", "ProcessInput.ObjectSt", "ProcessInput.ErrCoSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt", "ProcessInput.MotPosSt"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
        cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
        cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_export

        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

/**
Purpose: Export CPC_VP_TP Devices for S7_PLC front-end

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_VP_TP_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_TPConfig_ExportConfig(dsDpList, exceptionInfo);
}

/**
Purpose: Export CPC_VP_TP Devices for _UnPlc front-end

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_VP_TP_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_TPConfig_ExportConfig(dsDpList, exceptionInfo);
}

OPCUA_CPC_VP_TP_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_TPConfig_ExportConfig(dsDpList, exceptionInfo);
}

BACnet_CPC_VP_TP_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo) {
    CPC_VP_TPConfig_ExportConfig(dsDpList, exceptionInfo);
}

//@}
