/**@file

// cpcVR_GT.ctl
This library contains the widget, faceplate, etc. functions of VR_GT.

@par Creation Date
  dd/mm/yyyy

@par Modification History

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author
  Marcin Bes (BE-ICS)
  the author (DEP-GROUP)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved
*/
//@{

/** Returns the list of VR_GT DPE with alarm config that can be acknowledged

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name DP name
@param dpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
CPC_VR_GT_AcknowledgeAlarm(string deviceName, string dpType, dyn_string &dsNeedAck) {
//begin_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
    dsNeedAck = makeDynString(UN_ACKNOWLEDGE_PLC);
//end_TAG_SCRIPT_DEVICE_TYPE_AcknowledgeAlarm
}

/** Function called from snapshot utility of the treeDeviceOverview to get the time and value

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName  input, device name
@param deviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
CPC_VR_GT_ObjectListGetValueTime(string deviceName, string deviceType, dyn_string &dsReturnData) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
    float value;
    string sTime;

    //dpGet(deviceName + ".ProcessInput.PosSt", value,
    //      deviceName + ".ProcessInput.PosSt:_online.._stime", sTime);

    //dsReturnData[1] = sTime;
    //dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(deviceName + ".ProcessInput.PosSt"), value) + " " + dpGetUnit(deviceName + ".ProcessInput.PosSt"); // formated value with unit
//end_TAG_SCRIPT_DEVICE_TYPE_ObjectListGetValueTime
}

/** pop-up menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param dsAccessOk input, the access control
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
CPC_VR_GT_MenuConfiguration(string deviceName, string dpType, dyn_string dsAccessOk, dyn_string &menuList) {
    dyn_string exceptionInfo;
//begin_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT);

    // Add device context menu items
    unVacObject_addAliasActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    
    unVacObject_addDisableEnableActionToMenu(deviceName, dpType, dsAccessOk, menuList);
    int iPos = unVacObject_addVacuumExtensionActionToMenu(deviceName, dpType,  dsAccessOk, menuList);
    iPos = unVacObject_addProcessActionToMenu(deviceName, dpType,  dsMenuConfig, dsAccessOk, menuList, iPos+1);
//end_TAG_SCRIPT_DEVICE_TYPE_MenuConfiguration
}

/** handle the answer of the popup menu

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param dpType input, device type
@param menuList input, the access control
@param menuAnswer input, selected menu value
*/
CPC_VR_GT_HandleMenu(string deviceName, string dpType, dyn_string menuList, int menuAnswer) {
    dyn_string exceptionInfo;

    // The help file
    string helpFile = unVacObject_getHelpLink(dpType);
    
    if(!unVacObject_handleCustomUnicosMenu(deviceName, dpType, menuList, menuAnswer, helpFile, exceptionInfo)){
      cpcGenericObject_HandleUnicosMenu(deviceName, dpType, menuList, menuAnswer, exceptionInfo);
    }
    
    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click", exceptionInfo);
    }
}

/** Init static values which are used in widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VR_GT_WidgetInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetInitStatics
}

/** Returns the list of VR_GT DPEs which should be connected on widget animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VR_GT_WidgetDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs

    dynAppend(dpes, deviceName + ".eventMask");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");

//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VR_GT_WidgetAnimation(dyn_string dpes, dyn_anytype values, string widgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation

    int iSystemIntegrityAlarmValue = values[1];
    bool bSystemIntegrityAlarmEnabled = values[2];
    bool bLocked = values[3];
    string sSelectedManager = values[4];
    bool bIsCommentsActive = values[5];
    
    int iMask = values[6];
    bit32 bit32StsReg01 = values[7];
    bool bStsReg01Invalid = values[8];
    
    string sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sSelectColor;
       
    // 1. Select
    bool bLockVisible;
    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sSelectColor, bLockVisible);

    // 2. Warning
    sWarningLetter = "";
    sWarningColor = "unFaceplate_WarningActive";

    if(!bStsReg01Invalid && getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1) {
      sWarningLetter = CPC_WIDGET_TEXT_WARNING_ERROR;
    } else if(!bStsReg01Invalid && getBit(bit32StsReg01, CPC_StsReg01_IOSIMUW) == 1) {
      sWarningLetter = CPC_WIDGET_TEXT_WARNING_SIMU;
    }

    bool bInvalidLetter = bStsReg01Invalid;
    // 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
    // or the checking is not enabled then set the letter to O and the color to not valid data
    cpcGenericObject_WidgetValidnessAnimation(bSystemIntegrityAlarmEnabled, iSystemIntegrityAlarmValue, bInvalidLetter, sWarningLetter, sWarningColor);

    bool bInvalidBody = bStsReg01Invalid;

    // 3. Alarm

    // 4. control state
    sControlLetter = "";
    sControlColor = "unWidget_ControlStateManualLocal";
    
    dyn_string sBodyColor;
    // set color of body
    if (bInvalidBody) { // blue
      sBodyColor = "unDataNotValid";
    } else if((getBit(bit32StsReg01, CPC_StsReg01_IOERRORW) == 1)){
      sBodyColor = "cpcColor_Widget_Warning";
    } else { // normal green
      sBodyColor = "cpcColor_Widget_Status";
    }

    // 5. Animate all
    if (g_bSystemConnected) {
        setMultiValue("Body1",            "foreCol",  sBodyColor,
                      "Body2",            "foreCol",  sBodyColor,

                      "WarningText",      "text",     sWarningLetter, 
                      "WarningText",      "foreCol",  sWarningColor,

                      "AlarmText",        "text",  "",
                      "ControlStateText", "text",  "",

                      "commentsButton",   "visible",  bIsCommentsActive,
                      "SelectArea",       "foreCol",  sSelectColor, 
                      "LockBmp",          "visible",  bLockVisible,
                      "WidgetArea",       "visible",  true,

                      "eventState",       "visible",  iMask == 0

                      );
          
    }
    
    
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetAnimation
}

/** Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
CPC_VR_GT_WidgetDisconnection(string sWidgetType) {
//begin_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
    setMultiValue("Body1",            "foreCol",  "unDataNoAccess",
                  "Body2",            "foreCol",  "unDataNoAccess",
                  "WarningText",      "text",     "", 
                  "AlarmText",        "text",     "", 
                  "ControlStateText", "text",     "",
                  "SelectArea",       "foreCol",  "", 
                  "LockBmp",          "visible",  false, 
                  "WidgetArea",       "visible",  false,
                  "eventState",       "visible",  false);
//end_TAG_SCRIPT_DEVICE_TYPE_WidgetDisconnection
}

/** Return button configuration including access level

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
mapping CPC_VR_GT_ButtonConfig(string deviceName) {
    mapping buttons;
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    dyn_string exceptionInfo;

    buttons[UN_FACEPLATE_BUTTON_SELECT]               = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_REFRESH]              = UN_ACCESS_RIGHTS_OPERATOR;
    buttons[UN_FACEPLATE_BUTTON_RESTORE_ALL_AND_WRITE]= UN_ACCESS_RIGHTS_EXPERT;
    buttons[CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE] = UN_ACCESS_RIGHTS_EXPERT;
    buttons[UN_FACEPLATE_MASKEVENT]                   = UN_ACCESS_RIGHTS_EXPERT;
    
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonConfig
    return buttons;
}

/** Configure the list of dpes that needs for buttons animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the name of device
@param dpes input, the list of dpes to connect
*/
CPC_VR_GT_ButtonDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonDPEs
}

/** Set the state of the contextual button of the device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param dpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
CPC_VR_GT_ButtonSetState(string deviceName, string dpType, dyn_string dsUserAccess, dyn_string dsData) {
    bool buttonEnabled, bSelected;
    string localManager;
    bool bLocked = dsData[1];
    string sSelectedUIManager = dsData[2];

    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
    bSelected = (localManager == "S"); // Selection state
    dyn_string dsButtons = unSimpleAnimation_ButtonList(deviceName);

//begin_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
    for (int i = 1; i <= dynlen(dsButtons); i++) {
        buttonEnabled = bSelected && (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
        if (dsButtons[i] == UN_FACEPLATE_BUTTON_SELECT) {
            unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
        }
        else {
            cpcButton_setButtonState(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], buttonEnabled);
        
        }
    }
//end_TAG_SCRIPT_DEVICE_TYPE_ButtonSetState
}

/** Init static values which are used in faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name

*/
CPC_VR_GT_FaceplateInitStatics(string deviceName) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
    dyn_string exceptionInfo;
    
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.Rel1St"), g_params["validStatusLabels"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.ObjectSt"), g_params["objectStLabels"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.CommSt"), g_params["commStLabels"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.G1TFiltSt"), g_params["tFilter_mapping"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.G1PrSourSt"), g_params["source_mapping"]);
    cpcGenericObject_StringToMapping(cpcGenericDpFunctions_getMapping(deviceName, ".ProcessInput.GPennLCoSt"), g_params["GPennLCo_Mapping"]);

    g_params["PressureFormat"]  = dpGetFormat(deviceName + ".ProcessInput.G1PrHSt");
    g_params["PressureUnit"]    = dpGetUnit(deviceName + ".ProcessInput.G1PrHSt");
    g_params["RelToffMinUnit"]  = dpGetUnit(deviceName + ".ProcessInput.Rel1ToffMinSt");
    g_params["RelToffMinFormat"]= "%6.1f";
    g_params["G1HWVarUnit"]     = dpGetUnit(deviceName + ".ProcessInput.G1HWVarSt");
    g_params["G2HWVarUnit"]     = dpGetUnit(deviceName + ".ProcessInput.G2HWVarSt");
    g_params["G3HWVarUnit"]     = dpGetUnit(deviceName + ".ProcessInput.G3HWVarSt");
    g_params["G4HWVarUnit"]     = dpGetUnit(deviceName + ".ProcessInput.G4HWVarSt");

    g_params["G1"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_GAUGE1_NAME, exceptionInfo);
    g_params["G2"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_GAUGE2_NAME, exceptionInfo);
    g_params["G3"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_GAUGE3_NAME, exceptionInfo);
    g_params["G4"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_GAUGE4_NAME, exceptionInfo);
    g_params["R1"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY1_NAME, exceptionInfo);
    g_params["R2"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY2_NAME, exceptionInfo);
    g_params["R3"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY3_NAME, exceptionInfo);
    g_params["R4"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_RELAY4_NAME, exceptionInfo);

    //setDollarParams("PANEL_REF28",makeDynString("$bG3active"), makeDynString("true"));
    
    //g_params["GaugeType"] = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_VR_GT_GAUGE_TYPE, exceptionInfo);
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateInitStatics
}

/** Returns the list of VR_GT DPEs which should be connected on faceplate animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device name
@param dpes input/output, dpe list

*/
CPC_VR_GT_FaceplateDPEs(string deviceName, dyn_string &dpes) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
    dynAppend(dpes, deviceName + ".eventMask");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01");
    dynAppend(dpes, deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt");
    dynAppend(dpes, deviceName + ".ProcessInput.ObjectSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.CommSt");
    dynAppend(dpes, deviceName + ".ProcessInput.CommSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4St");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PrG1St");
    dynAppend(dpes, deviceName + ".ProcessInput.PrG1St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.PrG3St");
    dynAppend(dpes, deviceName + ".ProcessInput.PrG3St:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G1TFiltSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G1TFiltSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G2TFiltSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G2TFiltSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G3TFiltSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G3TFiltSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G4TFiltSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G4TFiltSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrSourSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrSourSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrSourSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrSourSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrHSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrHSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrHSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrHSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrLSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G1PrLSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrLSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G3PrLSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.GPennLCoSt");
    dynAppend(dpes, deviceName + ".ProcessInput.GPennLCoSt:_online.._invalid");

    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ScrSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ScrSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ScrSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ScrSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ScrSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ScrSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ScrSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ScrSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1HSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1HSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2HSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2HSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3HSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3HSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4HSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4HSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1LSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1LSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2LSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2LSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3LSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3LSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4LSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4LSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.FirmVersSt");
    dynAppend(dpes, deviceName + ".ProcessInput.FirmVersSt:_online.._invalid");
    
    dynAppend(dpes, deviceName + ".ProcessInput.G1HWVarSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G1HWVarSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G2HWVarSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G2HWVarSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G3HWVarSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G3HWVarSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.G4HWVarSt");
    dynAppend(dpes, deviceName + ".ProcessInput.G4HWVarSt:_online.._invalid");

    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ToffMinSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel1ToffMinSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ToffMinSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel2ToffMinSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ToffMinSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel3ToffMinSt:_online.._invalid");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ToffMinSt");
    dynAppend(dpes, deviceName + ".ProcessInput.Rel4ToffMinSt:_online.._invalid");
    
//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateDPEs
}

/** callback function on the device data

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dpes input, the dpe names
@param values input, the dpe values

*/
CPC_VR_GT_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values) {
//begin_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
    unFaceplate_animateTranslatedValue(dpes, values, "ObjectSt", "", "", g_params["objectStLabels"], "cpcColor_Faceplate_Status");

    unFaceplate_animateTranslatedValue(dpes, values, "CommSt", "", "", g_params["commStLabels"], "cpcColor_Faceplate_Status");

    unFaceplate_animateTranslatedValue(dpes, values, "Rel1St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel2St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel3St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel4St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
    

    unFaceplate_animateTranslatedValue(dpes, values, "Rel1ScrSt", "", "", g_params["source_mapping"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel2ScrSt", "", "", g_params["source_mapping"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel3ScrSt", "", "", g_params["source_mapping"], "cpcColor_Faceplate_Status");
    unFaceplate_animateTranslatedValue(dpes, values, "Rel4ScrSt", "", "", g_params["source_mapping"], "cpcColor_Faceplate_Status");

    unFaceplate_animateOnlineValue(dpes, values, "FirmVersSt", "", "%2d", "cpcColor_Faceplate_Status");  // mapping?

    unFaceplate_animateOnlineValue(dpes, values, "Rel1HSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel2HSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel3HSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel4HSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel1LSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel2LSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel3LSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel4LSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
    unFaceplate_animateOnlineValue(dpes, values, "Rel1ToffMinSt", g_params["RelToffMinUnit"], g_params["RelToffMinFormat"], "cpcColor_Faceplate_Status");   // format + unit?
    unFaceplate_animateOnlineValue(dpes, values, "Rel2ToffMinSt", g_params["RelToffMinUnit"], g_params["RelToffMinFormat"], "cpcColor_Faceplate_Status");   // format + unit?
    unFaceplate_animateOnlineValue(dpes, values, "Rel3ToffMinSt", g_params["RelToffMinUnit"], g_params["RelToffMinFormat"], "cpcColor_Faceplate_Status");   // format + unit?
    unFaceplate_animateOnlineValue(dpes, values, "Rel4ToffMinSt", g_params["RelToffMinUnit"], g_params["RelToffMinFormat"], "cpcColor_Faceplate_Status");   // format + unit?

    // Load the reference alias
    string refAlias;
    if(isDollarDefined("$sRefDpName") && $sRefDpName != ""){
        refAlias = unGenericDpFunctions_getAlias($sRefDpName);
        refAlias = substr(refAlias, strpos(refAlias, ":")+1);
    }      

    // Animates the relays    
    string sRelay, sRelayColor;
    for(int i = 1; i<=4; i++){
      sRelay = "R"+i;
      
      // Check whether connected
      if(g_params[sRelay] != ""){
         // Base the relay color on the reference alias. Red if the reference is coming from an VR_AH
         sRelayColor = g_params[sRelay] == refAlias ? "red" : "black";
         // Set the name and layout
         setMultiValue(sRelay+"Name", "toolTipText", g_params[sRelay], sRelay+"Name", "foreCol", sRelayColor);     
       } else {
         setMultiValue(sRelay+"Name", "enabled", false);              
       }         
    }   
  
    
    if(g_params["G1"] != ""){     
      //unFaceplate_animateText("G1Name", g_params["G1"], "unDisplayValue_Status");
      setValue("G1Name", "toolTipText", g_params["G1"]);
      if(g_params["G1"] == refAlias){
        setValue("G1Name", "foreCol", "red");
      }
      unFaceplate_animateTranslatedValue(dpes, values, "PrG1St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
      unFaceplate_animateTranslatedValue(dpes, values, "G1TFiltSt", "", "", g_params["tFilter_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateTranslatedValue(dpes, values, "G1PrSourSt", "", "", g_params["source_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G1PrHSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G1PrLSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G1HWVarSt", g_params["G1HWVarUnit"], "", "cpcColor_Faceplate_Status");   // format + unit?
    } else {
      setMultiValue("PrG1St",     "enabled", false,
                    "G1Name",     "enabled", false,
                    "G1TFiltSt",  "enabled", false,
                    "G1PrSourSt", "enabled", false,
                    "G1PrHSt",    "enabled", false,
                    "G1PrLSt",    "enabled", false,
                    "G1HWVarSt",  "enabled", false
                    );
    }

    if(g_params["G2"] != ""){
      //unFaceplate_animateText("G2Name", g_params["G2"], "unDisplayValue_Status");
      setValue("G2Name", "toolTipText", g_params["G2"]);
      if(g_params["G2"] == refAlias){
        setValue("G2Name", "foreCol", "red");
      }
      unFaceplate_animateTranslatedValue(dpes, values, "G2TFiltSt", "", "", g_params["tFilter_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G2HWVarSt", g_params["G2HWVarUnit"], "", "cpcColor_Faceplate_Status");   // format + unit?
    } else {
      setMultiValue("G2Name",     "enabled", false,
                    "G2TFiltSt",  "enabled", false,
                    "G2HWVarSt",  "enabled", false
                    );
    }

    if(g_params["G3"] != ""){
      if(g_params["G3"] == refAlias){
        setValue("G3Name", "foreCol", "red");
      }
      unFaceplate_animateTranslatedValue(dpes, values, "PrG3St", "", "", g_params["validStatusLabels"], "cpcColor_Faceplate_Status");
      //unFaceplate_animateText("G3Name", g_params["G3"], "unDisplayValue_Status");
      setValue("G3Name", "toolTipText", g_params["G3"]);
      unFaceplate_animateTranslatedValue(dpes, values, "G3TFiltSt", "", "", g_params["tFilter_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateTranslatedValue(dpes, values, "G3PrSourSt", "", "",  g_params["source_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G3PrHSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G3PrLSt", g_params["PressureUnit"], g_params["PressureFormat"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G3HWVarSt", g_params["G3HWVarUnit"], "", "cpcColor_Faceplate_Status");   // format + unit?
    } else {
      setMultiValue("PrG3St",     "enabled", false,
                    "G3Name",     "enabled", false,
                    "G3TFiltSt",  "enabled", false,
                    "G3PrSourSt", "enabled", false,
                    "G3PrHSt",    "enabled", false,
                    "G3PrLSt",    "enabled", false,
                    "G3HWVarSt",  "enabled", false
                    );
    }

    if(g_params["G4"] != ""){
      if(g_params["G4"] == refAlias){
        setValue("G4Name", "foreCol", "red");
      }
      //unFaceplate_animateText("G4Name", g_params["G4"], "unDisplayValue_Status");
      setValue("G4Name", "toolTipText", g_params["G4"]);
      unFaceplate_animateTranslatedValue(dpes, values, "G4TFiltSt", "", "", g_params["tFilter_mapping"], "cpcColor_Faceplate_Status");
      unFaceplate_animateOnlineValue(dpes, values, "G4HWVarSt", g_params["G4HWVarUnit"], "", "cpcColor_Faceplate_Status");   // format + unit?
    } else {
      setMultiValue("G4Name",     "enabled", false,
                    "G4TFiltSt",  "enabled", false,
                    "G4HWVarSt",  "enabled", false
                    );
    }

    if(g_params["G1"] != "" || g_params["G3"] != ""){
      unFaceplate_animateTranslatedValue(dpes, values, "GPennLCoSt", "", "", g_params["GPennLCo_Mapping"], "cpcColor_Faceplate_Status");
    } else {
        setMultiValue("GPennLCoSt",     "enabled", false);
    }

    cpcGenericObject_animateMaskEvent(dpes, values);

    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOERRORW", true, "cpcColor_Faceplate_Warning");
    unFaceplate_animateStatusBit(dpes, values, "CPC_StsReg01_IOSIMUW", true, "cpcColor_Faceplate_Warning");

//end_TAG_SCRIPT_DEVICE_TYPE_FaceplateStatusAnimationCB
}

//@}
