/**
  This script used to import importation configurations for ASCII Manager which allows to export data.

  Installation instruction.
  1. Copy the script to the script folder of your pvss project.
  2. If it is necessary configure constant PATH_PREFIX.
  3. Run the script via PVSS Console or OS terminal:
	a. in the PVSS Console add 'Control Manager' with script name as an options and put 'start mode' to 'manual'; then start this new manager once.
	b. in the OS console (windows cmd or linux terminal) execure follow command:
		 PVSS00ctrl -proj [PROJECT_NAME] unCPC_AsciiExportConfigurations.ctl
	   where PROJECT_NAME is name of your pvss project
*/

main()
{
  string PATH_PREFIX = getPath(DPLIST_REL_PATH);
  import_dp_configuration("VACUUM_generated", PATH_PREFIX + "VACUUM.dpl",
                          makeDynBool(1, 0, 0, 0, 0, 0),
                          makeDynString("alle"),
                          makeDynString("CPC_VG_PT", "CPC_VP_TP", "CPC_VR_GT"),
                          makeDynString("alle"),
                          makeDynString("alle")
                          );
  import_dp_configuration("VACUUMinfo_generated", PATH_PREFIX + "VACUUMinfo.dpl",
                          makeDynBool(0, 1, 0, 1, 0, 0),
                          makeDynString("_template_unCPC_VG_PT_Trends", "CPC_VG_PT_trendConfiguration", "CPC_VG_PT_unicosInfo",
										"_template_unCPC_VP_TP_Trends", "CPC_VP_TP_trendConfiguration", "CPC_VP_TP_unicosInfo",
                                        "_template_unCPC_VR_GT_Trends", "CPC_VR_GT_trendConfiguration", "CPC_VR_GT_unicosInfo"
                                        ),
                          makeDynString("alle"),
                          makeDynString("alle"),
                          makeDynString("alle")
                          );
  import_dp_configuration("VACUUMfw_generated", PATH_PREFIX + "VACUUMfwDeviceDefinition.dpl",
                          makeDynBool(0, 1, 0, 1, 0, 0),
                          makeDynString("CPC_VG_PTInfo", "CPC_VP_TPInfo", "CPC_VR_GTInfo"),
                          makeDynString("alle"),
                          makeDynString("alle"),
                          makeDynString("alle")
                          );
}

/**
  save import DP configuration for ASCII manager into the related dp

  parameters:
  cname - configuration name
  path - exported file path
  filters - arrays of 6 booleans for output filters:
    1 - datapoint types (T) - DPTypen
    2 - data points (D) - DP
    3 - aliases and comments (A) - Alias
    4 - original values (O) - Originalwerte
    5 - parametrization (P) - Parametrierung
    6 - parametrization including history (PH) - Historisch
  dp - datapoints that will be imported. array with 1 'alle' string means all elements will be imported
  dpt - datapoint types that will be imported. array with 1 'alle' string means all elements will be imported
  sprachen
  konfig
*/
import_dp_configuration(string cname, string path, dyn_bool filters, dyn_string dp, dyn_string dpt, dyn_string sprachen, dyn_string konfig) {
  string configDP = "";
  dyn_dyn_anytype allConfigs;

  // init config dpe
  // search for existing dpe
  dpQuery("SELECT '_original.._value' FROM 'o*.Name' WHERE _DPT = \"_AsciiConfig\"", allConfigs);
  for (int i = 2; i <= dynlen(allConfigs); i++) {
    if (allConfigs[i][2] == cname) {
      configDP = dpSubStr(allConfigs[i][1],DPSUB_DP);
      break;
    }
  }
  if (configDP == "") { // create new dpe
    int idx = 1;
    while (true) {
        configDP = "o_" + idx;
        if (dpExists(configDP)==0) {
          dpCreate(configDP, "_AsciiConfig");
          break;
        }
        idx += 1;
    }
  }

  // update element
  dpSet(configDP + ".Name:_original.._value",cname);

  dpSet(configDP + ".ASCII_Datei:_original.._value", path,
        configDP + ".OutFilter.DPTypen:_original.._value", filters[1],
        configDP + ".OutFilter.DP:_original.._value", filters[2],
        configDP + ".OutFilter.Alias:_original.._value", filters[3],
        configDP + ".OutFilter.Originalwerte:_original.._value", filters[4],
        configDP + ".OutFilter.Parametrierung:_original.._value", filters[5],
        configDP + ".OutFilter.Historisch:_original.._value", filters[6],
        configDP + ".Lokalzeit:_original.._value", 0,
        configDP + ".Sprachen.Mehrsprachig:_original.._value", 0,
        configDP + ".Sprachen.Sprachen:_original.._value",sprachen,
        configDP + ".DP:_original.._value",dp,
        configDP + ".DPTypen:_original.._value",dpt,
        configDP + ".Younger.Younger:_original.._value", 0,
        configDP + ".Younger.Zeit:_original.._value",getCurrentTime(),
        configDP + ".Konfig:_original.._value",konfig);

  dpSet(configDP + ".outputVersion:_original.._value", 0);
}
